/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypDev.
 HypDev is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#include"hyp.dev.h"
#include"hyp.ker.mainparameters.h"


struct CRegionData
{
	CRegionData() { m_Rx=m_Ry=m_Rz=0.0f; m_bRx=m_bRy=m_bRz=false; }
	t_real m_Rx;
	t_real m_Ry;
	t_real m_Rz;
	bool m_bRx;
	bool m_bRy;
	bool m_bRz;

	virtual void Show(std::ostream&) const;
};

struct CForceData :
public CRegionData
{
	CForceData() { m_Fx=m_Fy=m_Fz=0.0f; }
	t_real m_Fx;
	t_real m_Fy;
	t_real m_Fz;

	virtual void Show(std::ostream&) const;
};

struct CDOFData :
public CRegionData
{
	CDOFData() { m_bDOFx=m_bDOFy=m_bDOFz=false; }
	bool m_bDOFx;
	bool m_bDOFy;
	bool m_bDOFz;

	virtual void Show(std::ostream&) const;
};

struct CMeshingData
{
	CMeshingData() { m_DimX=m_DimY=m_DimZ=10; m_NbElementX=m_NbElementY=m_NbElementZ=5; }
	t_real m_DimX;
	t_real m_DimY;
	t_real m_DimZ;
	int m_NbElementX;
	int m_NbElementY;
	int m_NbElementZ;

	virtual void Show(std::ostream&) const;
};
	

void ShowLabel(std::ostream& Stream,const std::string& label,const std::string& descrip)
{
	Stream<<"------------------------------------------------"<<"\n"
		  <<descrip<<"["<<label<<"]\n";
}


void CRegionData::Show(std::ostream& Stream) const
{
	Stream<<"Y-Z plan equation, X = "<<m_Rx<<"\n"
		  <<"X-Z plan equation, Y = "<<m_Ry<<"\n"
		  <<"X-Y plan equation, Z = "<<m_Rz<<"\n"
		  <<"Y-Z plan selected : "<<m_bRx<<"\n"
		  <<"X-Z plan selected : "<<m_bRy<<"\n"
		  <<"X-Y plan selected : "<<m_bRz<<"\n";
}

void CForceData::Show(std::ostream& Stream) const
{
	ShowLabel(Stream,"-f","Forces");
	Stream<<"Force component belong X axe : "<<m_Fx<<"\n"
		  <<"Force component belong Y axe : "<<m_Fy<<"\n"
		  <<"Force component belong Z axe : "<<m_Fz<<"\n";
	CRegionData::Show(Stream);
}

void CDOFData::Show(std::ostream& Stream) const
{
	ShowLabel(Stream,"-dof","Degree of freedom");
	Stream<<"Fixing along the X axe : "<<m_bDOFx<<"\n"
	      <<"Fixing along the Y axe : "<<m_bDOFy<<"\n"
		  <<"Fixing along the Z axe : "<<m_bDOFz<<"\n";
	CRegionData::Show(Stream);
}

void CMeshingData::Show(std::ostream& Stream) const
{
	ShowLabel(Stream,"-g","Geometry of the cube");
	Stream<<"Dimension along the X axe : "<<m_DimX<<"\n"
	      <<"Dimension along the Y axe : "<<m_DimY<<"\n"
		  <<"Dimension along the Z axe : "<<m_DimZ<<"\n"
		  <<"Number of elements along the X axe : "<<m_NbElementX<<"\n"
	      <<"Number of elements along the Y axe : "<<m_NbElementY<<"\n"
		  <<"Number of elements along the Z axe : "<<m_NbElementZ<<"\n";
}

void ShowFile(std::ostream& Stream,const std::string* file)
{
	ShowLabel(Stream,"-xfile","Hyperion file"); Stream<<file[0]<<"\n";
	ShowLabel(Stream,"-txtfile","Txt file"); Stream<<file[1]<<"\n";
	ShowLabel(Stream,"-xfile","DirectX file"); Stream<<file[2]<<"\n";
	ShowLabel(Stream,"-logfile","Log file"); Stream<<file[3]<<"\n";
}

void ShowExceptionResult(std::ostream& Stream)
{
	Stream<<"See if you have put correct parameters or if you have enregistred the components.\n"
		  <<"If the problem still occurs, contact me by email at prebours@ema.fr.\n";
}



int main(int argc, char *argv[ ])
{
	typedef std::vector<CForceData> t_ForceDataEnum;
	typedef std::vector<CDOFData> t_DOFDataEnum;
	
	int i=0,j=0,k=0;
		
	bool bVerbose=false;
	bool bHelp=false;
	std::string file[4];
	t_ForceDataEnum ForceEnum;
	t_DOFDataEnum DOFEnum;
	CMeshingData MeshingData;
	std::ostream* pStream=0;
		
	CMainParameters p(argc,argv,'-');

	if(p.GetSize()==0) bHelp=true;

	for(i=0;i<p.GetSize();i++) {
		CLabelEnum e=p.GetValue(i);
		if(e.GetLabelName()=="defaultfiles") {
			file[0]=p.GetModulePath()+"hypdev.hyp";
			file[1]=p.GetModulePath()+"hypdev.txt";
			file[2]=p.GetModulePath()+"hypdev.x";
			file[3]=p.GetModulePath()+"hypdev.log";
		} else if(e.GetLabelName()=="hypfile") {
			CLabel l=e.GetValue(0);
			file[0]=l.GetValue(0);
		} else if(e.GetLabelName()=="txtfile") {
			CLabel l=e.GetValue(0);
			file[1]=l.GetValue(0);
		} else if(e.GetLabelName()=="xfile") {
			CLabel l=e.GetValue(0);
			file[2]=l.GetValue(0);
		} else if(e.GetLabelName()=="logfile") {
			CLabel l=e.GetValue(0);
			file[3]=l.GetValue(0);
		} else if(e.GetLabelName()=="v") {
			bVerbose=true;					
		} else if(e.GetLabelName()=="help") {
			bHelp=true;			
		} else if(e.GetLabelName()=="f") {
			ForceEnum.resize(e.GetSize());
			for(j=0;j<e.GetSize();j++) {
				CLabel l=e.GetValue(j);
				l.Get(ForceEnum[j].m_Rx,0);
				l.Get(ForceEnum[j].m_Ry,1);
				l.Get(ForceEnum[j].m_Rz,2);
				l.Get(ForceEnum[j].m_bRx,3);
				l.Get(ForceEnum[j].m_bRy,4);
				l.Get(ForceEnum[j].m_bRz,5);
				l.Get(ForceEnum[j].m_Fx,6);
				l.Get(ForceEnum[j].m_Fy,7);
				l.Get(ForceEnum[j].m_Fz,8);
				//				
			}
		} else if(e.GetLabelName()=="dof") {
			DOFEnum.resize(e.GetSize());
			for(j=0;j<e.GetSize();j++) {
				CLabel l=e.GetValue(j);
				l.Get(DOFEnum[j].m_Rx,0);
				l.Get(DOFEnum[j].m_Ry,1);
				l.Get(DOFEnum[j].m_Rz,2);
				l.Get(DOFEnum[j].m_bRx,3);
				l.Get(DOFEnum[j].m_bRy,4);
				l.Get(DOFEnum[j].m_bRz,5);
				l.Get(DOFEnum[j].m_bDOFx,6);
				l.Get(DOFEnum[j].m_bDOFy,7);
				l.Get(DOFEnum[j].m_bDOFz,8);
			}
		} else if(e.GetLabelName()=="g") {
			CLabel l=e.GetValue(0);
			l.Get(MeshingData.m_DimX,0);
			l.Get(MeshingData.m_DimY,1);
			l.Get(MeshingData.m_DimZ,2);
			l.Get(MeshingData.m_NbElementX,3);
			l.Get(MeshingData.m_NbElementY,4);
			l.Get(MeshingData.m_NbElementZ,5);
		}
	}

	if(!file[3].empty()) {
		hyp_ker::CDir dir;
		std::string full_path(file[3]);
		int pos=full_path.find_last_of('\\');
		//MODIF
		if ( pos!=std::string::npos ) { 
			dir.Open(std::string(full_path,0,pos));
			dir.CreateFile(std::string(full_path,pos+1,std::string::npos));
			pStream=new std::ofstream(file[3].c_str());
		} else {
			pStream=&(std::cout);
		}
	} else {
		pStream=&(std::cout);
	}
	
	
	*pStream<<hyp_ker::GetLibraryVersion()<<"\n\n";

	if(bHelp) {
		ShowLabel(*pStream,"-help","Help about optional parameters.");
		*pStream<<"-g double double double int int int\n"
			<<"-dof double double double bool bool bool bool bool bool\n"
			<<"-f double double double bool bool bool double double double\n"
			<<"-xfile char*\n"
			<<"-hypfile char*\n"
			<<"-txtfile char*\n"
			<<"-logfile char*\n"
			<<"-defaultfiles\n"
			<<"-help\n"
			<<"-v\n"
			<<"See the documentation for more information : http://lagrandeporte.multimania.com/hyperion\n\n";
	}

	if(bVerbose) {
		*pStream<<"\n"<<p.GetLineOfParameters()<<"\n";
		ShowFile(*pStream,file);
		for(i=0;i<ForceEnum.size();i++) {
			ForceEnum[i].Show(*pStream);			
		}
		for(i=0;i<DOFEnum.size();i++) {
			DOFEnum[i].Show(*pStream);
		}
		MeshingData.Show(*pStream);
	}
	
	
	try{
		*pStream<<"\n\nBeginning of computing->\n";
		CDevMesh mesh(*pStream);
		mesh.CreateMesh();
		
		mesh.Generation(MeshingData.m_DimX,
			MeshingData.m_DimY,
			MeshingData.m_DimZ,
			MeshingData.m_NbElementX,
			MeshingData.m_NbElementY,
			MeshingData.m_NbElementZ);
		
		for(i=0;i<DOFEnum.size();i++) {
			mesh.SetDOF(DOFEnum[i].m_Rx,
				DOFEnum[i].m_Ry,
				DOFEnum[i].m_Rz,
				DOFEnum[i].m_bRx,
				DOFEnum[i].m_bRy,
				DOFEnum[i].m_bRz,
				DOFEnum[i].m_bDOFx,
				DOFEnum[i].m_bDOFy,
				DOFEnum[i].m_bDOFz);
		}
		
		mesh.Construct();
		
		if(!file[0].empty()) mesh.SaveMesh(file[0]);

		for(i=0;i<ForceEnum.size();i++) {
			mesh.SetForce(ForceEnum[i].m_Rx,
				ForceEnum[i].m_Ry,
				ForceEnum[i].m_Rz,
				ForceEnum[i].m_bRx,
				ForceEnum[i].m_bRy,
				ForceEnum[i].m_bRz,
				ForceEnum[i].m_Fx,
				ForceEnum[i].m_Fy,
				ForceEnum[i].m_Fz);
		}

		mesh.Solve();

		if(!file[1].empty()) mesh.ShowMesh(file[1]);

		if(!file[2].empty()) mesh.SaveDisplayFile(file[2]);

	} catch(hyp_ker::CComException excep) {
		*pStream<<"!!!Hyperion Exception!!!"<<excep.What();
		ShowExceptionResult(*pStream);
	} catch(std::exception excep) {
		*pStream<<"!!!Standard Exception!!!"<<excep.what();
		ShowExceptionResult(*pStream);
	} catch(...) {
		*pStream<<"!!!Generic Exception!!!";
		ShowExceptionResult(*pStream);
	}

	*pStream<<"<-End of computing\n\n";

	if(bVerbose) {	
		*pStream<<"Libraries loaded...\n";
		hyp_ker::GetLoadedLibraries(*pStream);
	}
	
	*pStream<<"Unload of unused libraries...\n";
	hyp_ker::FreeUnusedLibraries();

	if(bVerbose) {
		*pStream<<"Libraries still used...\n";
		hyp_ker::GetLoadedLibraries(*pStream);
	}

	if(pStream!=0 && pStream!=&std::cout) {
		delete pStream;
	}

	return 0;
}