/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypDev.
 HypDev is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#include"hyp.dev.h"


void InvertEnum(t_label_enum& e)
{
	hyp_ASSERT(e.size()==3);
	t_label_enum cpy(e);
	e[0]=cpy[2];
	e[1]=cpy[1];
	e[2]=cpy[0];
	/*
	for(int i=0;i<e.size();i++) {
		e[i]=cpy[e.size()-1-i];
	}*/
}

void CDevMesh::CreateSide(const t_real& dim1,const t_real& dim2,const t_real& dim3,
				const t_size& div1,const t_size& div2,const t_size& div3,t_label& i)
{
	hyp_fem::IGeometricVertex* p_vertex=0;
 	hyp::t_label_enum e;

	for(t_real c1=0;c1<dim1;c1+=dim1/div1) {
		for(t_real c2=0;c2<dim2;c2+=dim2/div2) {
			for(t_real c3=0;c3<=dim3;c3+=dim3) {
				p_vertex=m_spBaseMesh->GetVertex(c1+dim1/div1,c2+dim2/div2,c3);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(c1+dim1/div1,c2,c3);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(c1,c2,c3);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				if(c3==dim3) InvertEnum(e);
				m_spObjectMesh->CreateSide(i,e);
				e.clear();i++;

				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(c1,c2+dim2/div2,c3);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(c1+dim1/div1,c2+dim2/div2,c3);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				if(c3==dim3) InvertEnum(e);
				m_spObjectMesh->CreateSide(i,e);
				e.clear();i++;
			}
		}
	}
}

/*
CDevMesh::Chrono::Chrono()
{
	hyp_ker::Clear(m_Profiler);
//	wf_START(m_Profiler);
}

CDevMesh::Chrono::~Chrono()
{
	wf_STOP(m_Profiler);
	t_int64 d=hyp_ker::Duration(m_Profiler);
	std::cout<<"CPU Cycle :"
		<<hyp::GetHighFromInt64(d)<<"*2^32"
		<<hyp::GetLowFromInt64(d)<<"\n";
}
*/


CDevMesh::~CDevMesh()
{

}

void CDevMesh::CreateMesh(void)
{
	m_spUnknownMesh.CreateInstance(hyp_fem::CLSID_hypCFEOMesh,0);
	InitMesh(m_spUnknownMesh);
}

void CDevMesh::InitMesh(hyp_ker::IUnknown* p_mesh)
{
	m_spUnknownMesh=p_mesh;
	m_spMeshMesh=m_spObjectMesh=m_spBaseMesh=m_spUnknownMesh;
}

void CDevMesh::ShowMesh(const std::string& path)
{
	m_Stream<<"Show Mesh in "<<path<<"...\n";
	hyp_out::t_spShowMeshObject spShow;
	spShow.CreateInstance(hyp_out::CLSID_hypCObjectStdFile,0);
	
	spShow->SetOutput(path);
	spShow->SetObject(m_spUnknownMesh);
	spShow->ShowAll();
	//divers
	spShow->ShowForceMask();
	spShow->ShowForces();

	spShow->ShowRenderMask();
	spShow->ShowDisplacements();

	spShow->CloseOutput();
}

void CDevMesh::SaveMesh(const std::string& path)
{
	m_Stream<<"Save Mesh in "<<path<<"...\n";
	hyp_out::t_spD3DMeshObject spD3DMesh;
	spD3DMesh.CreateInstance(hyp_out::CLSID_hypCObjectXFile,0);

	spD3DMesh->SetObject(m_spUnknownMesh);
	spD3DMesh->SaveD3DHypMesh(path);
}

void CDevMesh::SaveDisplayFile(const std::string& path)
{
	m_Stream<<"Save Display File in "<<path<<"...\n";
	hyp_out::t_spD3DMeshObject spD3DMesh;
	spD3DMesh.CreateInstance(hyp_out::CLSID_hypCObjectXFile,0);

	spD3DMesh->SetObject(m_spUnknownMesh);
	spD3DMesh->SaveD3DMeshBuilder(path);	
}

void CDevMesh::LoadMesh(const std::string& path)
{
	m_Stream<<"Load Mesh from "<<path<<"...\n";
	hyp_out::t_spD3DMeshObject spD3DMesh;
	spD3DMesh.CreateInstance(hyp_out::CLSID_hypCObjectXFile,0);

	spD3DMesh->SetObject(m_spUnknownMesh);
	spD3DMesh->LoadD3DHypMesh(path);
}

void CDevMesh::SetDOF(const t_real& x,const t_real& y,const t_real& z,
					  bool bx,bool by,bool bz,
					  bool dx,bool dy,bool dz)
{
	m_Stream<<"Apply DOF...\n";
	m_spMeshMesh->FixDOF(x,y,z,bx,by,bz,dx,dy,dz);
}

void CDevMesh::SetForce(const t_real& x,const t_real& y,const t_real& z,
		bool bx,bool by,bool bz,
		const t_real& fx,const t_real& fy,const t_real& fz)
{
	m_Stream<<"Apply Forces...\n";
	m_spMeshMesh->SetForce(x,y,z,bx,by,bz,fx,fy,fz);
}

void CDevMesh::Construct(void)
{
	m_Stream<<"Construct Global Matrix...\n";
	m_spMeshMesh->ConstructGlobalMatrix();
	m_Stream<<"Construct Boundary Matrix...\n";
	m_spMeshMesh->ConstructBoundaryMatrix();
	m_Stream<<"Inverse Boundary Matrix...\n";
	m_spMeshMesh->InverseBoundaryMatrix();
	m_Stream<<"Construct Visible Matrix...\n";
	m_spMeshMesh->ConstructVisibleMatrix();
}

void CDevMesh::Solve(void)
{
	m_Stream<<"Render...\n";
	m_spMeshMesh->Render();
	m_Stream<<"PostRender...\n";
	m_spMeshMesh->PostRender();
}

void CDevMesh::Generation(const t_real& a,const t_real& b,const t_real& c,
							const int& d_a,const int& d_b,const int& d_c)
{
	hyp::t_label i=0;
 	hyp::t_label_enum e;
	hyp::t_real xi,yi,zi;

	hyp_fem::IGeometricVertex* p_vertex=0;
	
	m_Stream<<"Creation of nodes...\n";
	i=0;
	for(xi=0;xi<=a;xi+=a/d_a) {
		for(yi=0;yi<=b;yi+=b/d_b) {
			for(zi=0;zi<=c;zi+=c/d_c) {
				hyp_ker::cerr<<xi<<","<<yi<<","<<zi<<"\n";
				m_spObjectMesh->CreateVertex(i,xi,yi,zi);
				i++;
			}
		}
	}

	m_Stream<<"Creation of elements...\n";
	i=0;
	for(xi=0;xi<a;xi+=a/d_a) {
		for(yi=0;yi<b;yi+=b/d_b) {
			for(zi=0;zi<c;zi+=c/d_c) {
				hyp_ker::cerr<<xi<<","<<yi<<","<<zi<<"\n";

				hyp_ker::cerr<<"0";
				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));
		
				hyp_ker::cerr<<"1";
				p_vertex=m_spBaseMesh->GetVertex(xi+a/d_a,yi,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				hyp_ker::cerr<<"2";
				p_vertex=m_spBaseMesh->GetVertex(xi+a/d_a,yi+b/d_b,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				hyp_ker::cerr<<"3";
				p_vertex=m_spBaseMesh->GetVertex(xi,yi+b/d_b,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				hyp_ker::cerr<<"4";
				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				hyp_ker::cerr<<"5";
				p_vertex=m_spBaseMesh->GetVertex(xi+a/d_a,yi,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				hyp_ker::cerr<<"6";
				p_vertex=m_spBaseMesh->GetVertex(xi+a/d_a,yi+b/d_b,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				hyp_ker::cerr<<"7\n";
				p_vertex=m_spBaseMesh->GetVertex(xi,yi+b/d_b,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				m_spMeshMesh->CreateElement(i,e);
				e.clear();
				i++;

			}
		}
	}

	m_Stream<<"Creation of sides...\n";
	i=0;
	CreateSide(a,b,c,d_a,d_b,d_c,i);
	//CreateSide(a,c,b,d_a,d_c,d_b,i);
	//CreateSide(b,c,a,d_c,d_b,d_a,i);


	for(xi=0;xi<a;xi+=a/d_a) {
		for(zi=0;zi<c;zi+=c/d_c) {
			for(yi=0;yi<=b;yi+=b) {
				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(xi+a/d_a,yi,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));
				
				p_vertex=m_spBaseMesh->GetVertex(xi+a/d_a,yi,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				if(yi==b) InvertEnum(e);
				m_spObjectMesh->CreateSide(i,e);
				e.clear();i++;
	
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				if(yi==b) InvertEnum(e);
				m_spObjectMesh->CreateSide(i,e);
				e.clear();i++;				
			}
		}
	}


	for(yi=0;yi<b;yi+=b/d_b) {
		for(zi=0;zi<c;zi+=c/d_c) {
			for(xi=0;xi<=a;xi+=a) {
				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));
			
				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(xi,yi+b/d_b,zi+c/d_c);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				if(xi==a) InvertEnum(e);
				m_spObjectMesh->CreateSide(i,e);
				e.clear();i++;

				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(xi,yi+b/d_b,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				p_vertex=m_spBaseMesh->GetVertex(xi,yi,zi);
				e.push_back(m_spBaseMesh->GetLabel(p_vertex));

				if(xi==a) InvertEnum(e);
				m_spObjectMesh->CreateSide(i,e);
				e.clear();
				i++;
			}
		}
	}
	
}
