/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypDev.
 HypDev is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.dev.h"
#include"hyp.ker.mainparameters.h"


int CArgData::m_Argc=0;
char** CArgData::m_Argv=0;

CLabel::CLabel(int PosBegin,int PosLast) :
m_PosBegin(PosBegin),
m_PosLast(PosLast)
{

}

int CLabel::GetSize(void) const
{
	return m_PosLast-m_PosBegin-1;
}

std::string CLabel::GetValue(int Pos) const
{
	if(Pos<GetSize())
		return m_Argv[m_PosBegin+Pos+1];
	else
		return std::string("");
}

int CLabelEnum::GetSize(void) const
{
	return size();
}

CLabel CLabelEnum::GetValue(int Pos) const
{
	if(Pos<GetSize())
		return at(Pos);
	else
		return CLabel();
}

CMainParameters::CMainParameters(int argc,char** argv,char LabelKey) :
m_LabelKey(LabelKey)
{
	SetArg(argc,argv);
}

int CMainParameters::GetSize(void) const
{
	int compt=0;
	int pos=1;
	while((pos=GetNextLabelPosition(pos))!=-1) {
		compt++;
		pos++;
	}
	return compt;
}

int CMainParameters::GetNextLabelPosition(int Pos) const
{
	for(int i=Pos;i<m_Argc;i++) {
		if(m_Argv[i][0]==m_LabelKey) return i;
	}
	return -1;
}

int CMainParameters::GetNextLabel(const std::string& LabelName,int Pos) const
{
	int pos=Pos;
	while((pos=GetNextLabelPosition(pos))!=-1) {
		if(GetLabelName(pos)==LabelName) return pos;
		pos++;
	}
	return -1;
}

std::string CMainParameters::GetLabelName(int Pos) const
{
	if(Pos<m_Argc && m_Argv[Pos][0]==m_LabelKey) return &m_Argv[Pos][1];
	else return 0;
}

CLabelEnum CMainParameters::GetValue(int RelativePos) const
{
	int compt=0;
	int pos=1;
	while( ((pos=GetNextLabelPosition(pos))!=-1) && (compt!=RelativePos) ) {
		compt++;
		pos++;
	}

	CLabelEnum LabelEnum(GetLabelName(pos));
		
	int pos_begin=GetNextLabel(LabelEnum.GetLabelName(),1);
	int pos_last=1;
	for(int i=0;i<GetSize();i++) {
		if( (pos_last=GetNextLabelPosition(pos_begin+1))==-1) pos_last=m_Argc;
		LabelEnum.push_back(CLabel(pos_begin,pos_last));
		pos_begin=pos_last;
		if( (pos_begin=GetNextLabel(LabelEnum.GetLabelName(),pos_begin))==-1) break;
	}	

	return LabelEnum;
}

std::string CMainParameters::GetModuleFileName(void) const
{
	return m_Argv[0];
}

std::string CMainParameters::GetLineOfParameters(void) const
{
	std::string str_return;
	for(int i=1;i<m_Argc;i++) {
		str_return+=m_Argv[i];
		str_return+=" ";
	}
	return str_return;
}

std::string CMainParameters::GetModuleName(void) const
{
	std::string full_path(m_Argv[0]);
	int pos=full_path.find_last_of('\\');
	return std::string(full_path,pos+1,std::string::npos);
}

std::string CMainParameters::GetModulePath(void) const
{
	std::string full_path(m_Argv[0]);
	int pos=full_path.find_last_of('\\');
	return std::string(full_path,0,pos+1);
}