/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 11/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(common)

void CContainer::Container::_debug_container(void)
{
	int size__=m_ContainerBase.size();
	hyp_ker::cerr<<"Size : "<<size__<<"\n";
	for(t_ContBase::const_iterator itor=m_ContainerBase.begin();
	itor!=m_ContainerBase.end();
	itor++) {
		hyp_ker::cerr<<itor->m_pIUnknown<<" -- "<<itor->m_Label<<"\n";
		itor->m_pIUnknown->AddRef();
		itor->m_pIUnknown->Release();
	}
}


CContainer::Container::~Container()
{
	Clear();
}

void CContainer::Container::Insert(const t_label& label,hyp_ker::IUnknown* pUnknown)
{
	m_ContainerBase.push_back(Content(label,pUnknown));	
	pUnknown->AddRef();
}

CContainer::Container::t_ContBase::iterator CContainer::Container::TypeAlreadyExists(hyp_ker::IUnknown* component)
{
	for(t_ContBase::iterator itor=m_ContainerBase.begin();
	itor!=m_ContainerBase.end();
	itor++) {
		if(component==itor->m_pIUnknown) break;
	}
	return itor;
}	

CContainer::Container::t_ContBase::iterator CContainer::Container::KeyAlreadyExists(const t_label& key)
{
	for(t_ContBase::iterator itor=m_ContainerBase.begin();
	itor!=m_ContainerBase.end();
	itor++) {
		if(key==itor->m_Label) break;
	}
	return itor;
}

void CContainer::Container::Clear(void)
{
	for(t_ContBase::iterator itor=m_ContainerBase.begin();
	itor!=m_ContainerBase.end();
	itor++) {
		itor->m_pIUnknown->Release();
	}
	m_ContainerBase.clear();
}

void CContainer::Container::Erase(t_ContBase::iterator itor)
{
	hyp_ASSERT(0);
	itor->m_pIUnknown->Release();
	m_ContainerBase.erase(itor);
}

t_label CContainer::Container::GetMaximumLabel(void) const
{
	t_label label_max=-1;
	if(!m_ContainerBase.empty()) {
		label_max=m_ContainerBase.begin()->m_Label;
		for(t_ContBase::const_iterator itor=m_ContainerBase.begin();
		itor!=m_ContainerBase.end();
		itor++) {
			if(itor->m_Label>label_max) 
				label_max=itor->m_Label;
		}
	}
	return label_max;
}


		
CContainer::CContainer(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter),
m_Selection(0)
{
	hypfem_TRACE_CONSTRUCTION(("CContainer"));
}

CContainer::~CContainer()
{
	hypfem_TRACE_DESTRUCTOR(("~CContainer"));
}


hyp_ker::com_result CContainer::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIContainer) {
		return FinishQI(static_cast<IContainer*>(this),ppv);
	} else if(iid==IID_hypIContainerIterator) {
		return FinishQI(static_cast<IContainerIterator*>(this),ppv);
	} else if(iid==IID_hypIContainerKey) {
		return FinishQI(static_cast<IContainerKey*>(this),ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG



hyp_ker::com_result CContainer::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CContainer(pUnknownOuter);
	return hyp_ker::com_ok;
}



/*!
The inserted interface has not associated label, so the implementation 
finds the first label which is superior to the other keys in the container.
*/
void CContainer::Add(hyp_ker::IUnknown* pObject)
{
	hyp_ASSERT(ContentIsValid(pObject));
	
	if(m_Container.TypeAlreadyExists(pObject)!=m_Container.End()) {
		hyp_THROW_COM(hyp_ker::com_logic_in_arguments);
	} else {
		//on incremente de 1 par exemple
		m_Container.Insert(m_Container.GetMaximumLabel()+1,pObject);
	}
}

void CContainer::RemoveAll()
{
	m_Container.Clear();
}

t_size CContainer::GetSize() const
{
	return m_Container.Size();
}

void CContainer::Remove(hyp_ker::IUnknown* pObject)
{
	Container::t_ContBase::iterator itor=m_Container.TypeAlreadyExists(pObject);
	if(itor!=m_Container.End()) {
		m_Container.Erase(itor);
	}
}




void CContainer::Add(hyp_ker::IUnknown* pObject,const t_label& Label)
{
	hyp_ASSERT(ContentIsValid(pObject));
	
	Container::t_ContBase::iterator itor_cmpnt=m_Container.TypeAlreadyExists(pObject);
	Container::t_ContBase::iterator itor_key=m_Container.KeyAlreadyExists(Label);

	pObject->AddRef();

	if(itor_cmpnt!=m_Container.End()) {
		if(itor_key==m_Container.End()) {	//component exists && key doesn't
			m_Container.Erase(itor_cmpnt);	
		} else {							//component exists && key does
			hyp_ASSERT(itor_key==itor_cmpnt);
			return;
		}
	} else if(itor_key!=m_Container.End()) {	//key exists && component doesn't
		m_Container.Erase(itor_key);		
	}
	m_Container.Insert(Label,pObject);
	pObject->Release();
}

void CContainer::Remove(const t_label& Label)
{
	Container::t_ContBase::iterator itor=m_Container.KeyAlreadyExists(Label);
	if(itor!=m_Container.End()) {
		m_Container.Erase(itor);
	}
}

hyp_ker::IUnknown* CContainer::GetContent(const t_label& Label)
{
	Container::t_ContBase::iterator itor=m_Container.KeyAlreadyExists(Label);
	if(itor!=m_Container.End()) {
		return itor->m_pIUnknown;
	} else {
		hyp_THROW_COM(hyp_ker::com_logic_in_arguments);
		return 0;
	}
}

t_label CContainer::GetKey(hyp_ker::IUnknown* pObject)
{
	hyp_ASSERT(ContentIsValid(pObject));
	
	Container::t_ContBase::iterator itor=m_Container.TypeAlreadyExists(pObject);
	if(itor!=m_Container.End()) {
		return itor->m_Label;
	} else {
		hyp_THROW_COM(hyp_ker::com_logic_in_arguments);
		return 0;
	}
}

void CContainer::SetKey(hyp_ker::IUnknown* pObject,const t_label& Label)
{
	hyp_ASSERT(ContentIsValid(pObject));
	
	Container::t_ContBase::iterator itor_key=m_Container.KeyAlreadyExists(Label);
	if(itor_key!=m_Container.End()) {
		hyp_THROW_COM(hyp_ker::com_logic_in_arguments);
	} else {
		Container::t_ContBase::iterator itor_cmpnt=m_Container.TypeAlreadyExists(pObject);
		if(itor_cmpnt!=m_Container.End()) {
			itor_cmpnt->m_Label=Label;
		} else {
			hyp_THROW_COM(hyp_ker::com_logic_in_arguments);
		}
	}
}




hyp_ker::IUnknown* CContainer::SetBeginSelection()
{
	if(!m_Container.Empty()) {
		m_Selection=m_Container.Begin();
	} else {
		hyp_THROW_COM2(hyp_ker::com_notified, ("Container Empty") );
	}	
	return GetSelection();
}

hyp_ker::IUnknown* CContainer::GetSelection()
{
	return m_Selection->m_pIUnknown;
}

hyp_ker::IUnknown* CContainer::IncrSelection()
{
	m_Selection++;
	if(m_Selection!=m_Container.End()) {
		return GetSelection();
	} else {
		//on est arriv  la fin. Il faut renvoyer une exception non letale
		//il permet de simuler la fonction end() des containers de la stl
		hyp_THROW_COM(hyp_ker::com_notified);
		return 0;
	}
}


hyp_NAMESPACE_END(common)
hyp_NAMESPACE_END_HYPFEM