/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Creation 11/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_com_container_h_
#define _hyp_com_container_h_

#ifndef hypfem_INSIDE_INTERN
#error Inclusion not allowed (include hyp.fem.intern.h instead)
#endif


/*!
\ingroup Ephy_FEM_Common
\HypDefaultDescripComponent s IContainer, IContainerKey and IContainerIterator.
\HypCLSID CLSID_hypCContainer {1187D861-9F81-11d4-A08F-0080AD97951B}
\HypBeginInterf hyp.com.container.cpp
\HypEndInterf

\HypImplDetails

*/
class CContainer :
#ifdef hypfem_IDENTIFY_CONSTRUCTION_ON
protected CIDClass,
#endif
public IContainer,
public IContainerKey,
public IContainerIterator,
public hyp_ker::CUnknown
{
public:
	/*! \HypCommonFonct*/
	CContainer(hyp_ker::IUnknown*);
	virtual ~CContainer();
	hyp_DECLARE_IUNKNOWN
	virtual hyp_ker::com_result NondelegatingQueryInterface(const hyp_ker::hypIID&,void**);
	static hyp_ker::com_result CreateInstance(hyp_ker::IUnknown* ,hyp_ker::CUnknown**);
	/*!@}*/

	/*! \name IContainer Implementation
	@{
	*/
	virtual void Add(hyp_ker::IUnknown*);
	virtual void RemoveAll();
	virtual t_size GetSize() const;
	virtual void Remove(hyp_ker::IUnknown*);
	/*!@}*/

	/*! \name IContainerKey Implementation
	@{
	*/
	virtual	void Add(hyp_ker::IUnknown*,const t_label&);
	virtual	void Remove(const t_label&);
	virtual hyp_ker::IUnknown* GetContent(const t_label&);
	virtual t_label GetKey(hyp_ker::IUnknown*);
	virtual void SetKey(hyp_ker::IUnknown*,const t_label&);
	/*!@}*/

	/*! \name IContainerIterator Implementation
	@{
	*/
	virtual hyp_ker::IUnknown* SetBeginSelection();
	virtual	hyp_ker::IUnknown* GetSelection();
	virtual hyp_ker::IUnknown* IncrSelection();
	/*!@}*/

private:
	class Container
	{
	public:
		struct Content {	
			Content();/* :	m_Label(0), m_pIUnknown(0) {}*/
			Content(const t_label& label,hyp_ker::IUnknown* p) : m_Label(label),m_pIUnknown(p) {}
			Content(const Content& content) {
				m_pIUnknown=content.m_pIUnknown;m_Label=content.m_Label; }
			~Content() {}
			t_label m_Label;
			hyp_ker::IUnknown* m_pIUnknown;
		};		
		typedef std::vector<Content> t_ContBase;
	private:
		t_ContBase m_ContainerBase;
	
	public:
		//no defined
		Container(const Container&);
		
		Container() { m_ContainerBase.reserve(100); };
		~Container();		
		void Insert(const t_label&,hyp_ker::IUnknown*);
		t_ContBase::iterator TypeAlreadyExists(hyp_ker::IUnknown*);
		t_ContBase::iterator KeyAlreadyExists(const t_label&);
		void Clear(void);
		void Erase(t_ContBase::iterator);
		t_label GetMaximumLabel(void) const;
		inline t_ContBase::iterator Begin(void) { return m_ContainerBase.begin(); }
		inline t_ContBase::iterator End(void) { return m_ContainerBase.end(); }
		inline t_size Size(void) const { return m_ContainerBase.size(); }
		inline bool Empty(void) const { return m_ContainerBase.empty(); }
		void _debug_container(void);
		
	} m_Container;
	Container::t_ContBase::const_iterator m_Selection;

	/*!
	Checks some fondamental assertion on the state of a particular component.
	\param pObject Component which is checked
	\internal
	*/
	inline bool ContentIsValid(hyp_ker::IUnknown* pObject) {
		return (pObject!=0)&&((void*)(pObject)!=(void*)(this));
	}
	
};


#endif