/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation  15/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(common)

CFactoriesLibrary::t_Cont CFactoriesLibrary::m_Cache;


CFactoriesLibrary::CFactoriesLibrary(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter)
{
	hypfem_TRACE_CONSTRUCTION(("CFactoriesLibrary"));
}

CFactoriesLibrary::~CFactoriesLibrary()
{
	//on supprime les factories
	for(t_ContIterator itor=m_Cache.begin();
	itor!=m_Cache.end();
	itor++) {
		//itor->m_pFactory->LockServer(false);
		itor->m_pFactory->Release();
	}

	hypfem_TRACE_DESTRUCTOR(("~CFactoriesLibrary"));
}


hyp_ker::com_result CFactoriesLibrary::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIFactoriesLibrary) {
		return FinishQI(static_cast<IFactoriesLibrary*>(this),ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG


hyp_ker::com_result CFactoriesLibrary::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CFactoriesLibrary(pUnknownOuter);
	return hyp_ker::com_ok;
}



void CFactoriesLibrary::CreateInstance(const hyp_ker::hypCLSID& clsid,
									   hyp_ker::IUnknown* pIUnknownOuter,
									   t_ulong clsctx,
									   const hyp_ker::hypIID& iid,
									   void** ppv)
{
	//FIXTODO
	void* pv_prov=0;
	GetData(clsid)->m_pFactory->CreateInstance(pIUnknownOuter,iid,(void**)&pv_prov);
	*ppv=pv_prov;	
}


void CFactoriesLibrary::CreateInstance(const char* progid,
									   hyp_ker::IUnknown* pIUnknownOuter,
									   t_ulong clsctx,
									   const hyp_ker::hypIID& iid,
									   void** ppv)
{
	GetData(progid)->m_pFactory->CreateInstance(pIUnknownOuter,iid,(void**)&ppv);
}



//le progid dans le clsid est peut etre different du progid indiquant le clsid
CFactoriesLibrary::SFactoryData* CFactoriesLibrary::GetData(const std::string& ProgID)
{
	for(t_ContIterator itor=m_Cache.begin();
	itor!=m_Cache.end();
	itor++) {
		if(itor->m_ProgID==ProgID) {
			return itor;
		}
	}

	//il faut peut tre charger
	hyp_ker::hypCLSID clsid;
	try {
		hyp_ker::CRegistryUtility reg;
		clsid=reg.GetCLSIDFromProgID(ProgID);
	} catch(hyp_ker::CComException hr) {
		//pas de clsid associ au progid
		hyp_THROW_COM(hyp_ker::com_false);
	}

	hyp_ker::IClassFactory* p_IFactory=0;
	if( hyp_ker::Failed( hyp_ker::GetClassObject(clsid,hyp_ker::INPROC_SERVER,0,
		hyp_ker::IID_hypIClassFactory,(void**)p_IFactory) ) ) {
		hyp_THROW_COM(hyp_ker::com_false);		
	}

	m_Cache.push_back(SFactoryData(ProgID,clsid,p_IFactory));
	return &m_Cache.back();
}

CFactoriesLibrary::SFactoryData* CFactoriesLibrary::GetData(const hyp_ker::hypCLSID& CLSID)
{
	for(t_ContIterator itor=m_Cache.begin();
	itor!=m_Cache.end();
	itor++) {
		if(itor->m_CLSID==CLSID) {
			return itor;
		}
	}

	//ok, il faut charger le composant
	hyp_ker::IClassFactory* p_IFactory=0;
	hyp_ker::com_result hr=hyp_ker::GetClassObject(CLSID,hyp_ker::INPROC_SERVER,0,
		hyp_ker::IID_hypIClassFactory,(void**)&p_IFactory);
	if( hyp_ker::Failed(hr) ) {
		hyp_THROW_COM(hyp_ker::com_false);		
	}

	//on vite de dcharger la dll
	//p_IFactory->LockServer(true);

	std::string prog_id;
	try {
		hyp_ker::CRegistryUtility reg;
		prog_id=reg.GetProgIDFromCLSID(CLSID);
	} catch(hyp_ker::CComException hr) {
		//il n'y aura pas de progid attach au clsid
		prog_id.erase();
	}

	//on ajoute une nouvelle donne
	hypfem_TRACE( ("Ajout du composant %s.",prog_id.c_str()) );
	m_Cache.push_back(SFactoryData(prog_id,CLSID,p_IFactory));
	return &m_Cache.back();
}

hyp_NAMESPACE_END(common)
hyp_NAMESPACE_END_HYPFEM