/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(data)

CGeometricBase::CGeometricBase(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter)
{
	hypfem_TRACE_CONSTRUCTION(("CGeometricBase"));
}

CGeometricBase::~CGeometricBase()
{
	hypfem_TRACE_DESTRUCTOR(("~CGeometricBase"));
}

hyp_ker::com_result CGeometricBase::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIGeometricBase) {
		return FinishQI(static_cast<IGeometricBase*>(this),ppv);
	} else if(iid==IID_hypIContainerKey ||
		iid==IID_hypIContainerIterator ||
		iid==IID_hypIContainer) {
		return m_spUnknownCont->QueryInterface(iid,ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG

hyp_ker::com_result CGeometricBase::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CGeometricBase(pUnknownOuter);
	return hyp_ker::com_ok;
}


hyp_ker::com_result CGeometricBase::Init(void)
{
	try{
		CGlobalFactory::CreateInstance(CLSID_hypCContainer,GetOuterUnknown(),
			m_spUnknownCont.iid(),(void**)&m_spUnknownCont);
	} catch(hyp_ker::CComException hr) {
		return hyp_ker::com_false;
	}
	return hyp_ker::com_ok;
}

void CGeometricBase::FinalRelease(void)
{
	CUnknown::FinalRelease();
}


//IGeometricSide
t_size CGeometricBase::GetNumberOfVertices()
{
	t_spContainer spCont=m_spUnknownCont;
	return spCont->GetSize();
}

IGeometricVertex* CGeometricBase::GetVertex(const t_label& Label)
{
	t_spContainerKey spContKey=m_spUnknownCont;
	t_spGeometricVertex spVertex=spContKey->GetContent(Label);
	return spVertex;
}

IGeometricVertex* CGeometricBase::GetVertex(const t_real& X,const t_real& Y,const t_real& Z)
{
	t_spContainerIterator spContItor=m_spUnknownCont;
	t_spGeometricVertex spVertex=spContItor->SetBeginSelection();
	try{
		while(1) {
			if(X==spVertex->GetCoord(0) &&
				Y==spVertex->GetCoord(1) &&
				Z==spVertex->GetCoord(2)) {
				return spVertex;
			}
			spVertex=spContItor->IncrSelection();
		}
	} catch(hyp_ker::CComException error) {
		if(hyp_ker::Failed(error.GetResult())) {
			throw error;
		} else {
			hyp_THROW_COM2(hyp_ker::com_false, ("Unable to localise the vertex.") );			
		}
	}
	return 0;
}

t_label CGeometricBase::GetLabel(IGeometricVertex* pObject)
{
	t_spContainerKey spContKey=m_spUnknownCont;
	return spContKey->GetKey(pObject);
}

void CGeometricBase::AddVertex(IGeometricVertex* pObject,const t_label& Label)
{
	//on regarde s'il est compatible avec le filtre
	if(IsCompatibleWithFilter(pObject)) {
		t_spContainerKey spContKey=m_spUnknownCont;
		spContKey->Add(pObject,Label);
	} else {
		hyp_THROW_COM2(hyp_ker::com_false, ("Incompatible with the filter") );
	}
}

void CGeometricBase::DeleteVertex(const t_label& Label)
{
	t_spContainerKey spContKey=m_spUnknownCont;
	spContKey->Remove(Label);
}

IContainerKey* CGeometricBase::GetContainerOfVertices()
{
	t_spContainerKey spContKey=m_spUnknownCont;
	return spContKey;
}

bool CGeometricBase::IsCompatibleWithFilter(IGeometricVertex* pObject) const
{
	hyp_ker::com_result hr=hyp_ker::com_ok;
	hyp_ker::IUnknown* pv=0;
	for(t_ContainerForIID::const_iterator itor=m_Filter.begin();
	itor<m_Filter.end();
	itor++) {
		hr=pObject->QueryInterface(**itor,(void**)&pv);
		if(hyp_ker::Failed(hr)) {
			return false;
		}
	}
	return true;
}

void CGeometricBase::SetFilter(const hyp_ker::hypIID* pIID)
{
	m_Filter.push_back(pIID);
}

void CGeometricBase::ClearFilter()
{
	m_Filter.clear();
}


hyp_NAMESPACE_END(data)
hyp_NAMESPACE_END_HYPFEM