/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 14/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(data)


CGeometricObject::CGeometricObject(hyp_ker::IUnknown* pUnknownOuter,const hyp_ker::hypCLSID* p_clsid) :
CUnknown(pUnknownOuter),
m_pCLSID_Vertex(p_clsid)
{
	hypfem_TRACE_CONSTRUCTION(("CGeometricObject"));
}

CGeometricObject::~CGeometricObject()
{
	hypfem_TRACE_DESTRUCTOR(("~CGeometricObject"));
}


hyp_ker::com_result CGeometricObject::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIGeometricObject) {
		return FinishQI(static_cast<IGeometricObject*>(this),ppv);
	} else if(iid==IID_hypIGeometricBase ||
		iid==IID_hypIContainer ||
		iid==IID_hypIContainerKey ||
		iid==IID_hypIContainerIterator) {
		return m_spUnknownVertices->QueryInterface(iid,ppv);
	} /*else if(iid==IID_hypID3DRMMeshObject) {
		return FinishQI(static_cast<ID3DRMMeshObject*>(this),ppv);
	}*/ else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG

/*!
The new component will create \ref CGeometricVertex component when the method \ref CreateVertex will be called.
*/
hyp_ker::com_result CGeometricObject::CreateInstanceForGeometry(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CGeometricObject(pUnknownOuter,&CLSID_hypCGeometricVertex);
	return hyp_ker::com_ok;
}

/*!
The new component will create \ref CFEONode component when the method \ref CreateVertex will be called.
*/
hyp_ker::com_result CGeometricObject::CreateInstanceForFiniteElement(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CGeometricObject(pUnknownOuter,&CLSID_hypCFEONode);
	return hyp_ker::com_ok;
}


hyp_ker::com_result CGeometricObject::Init(void)
{
	try{
		CGlobalFactory::CreateInstance(CLSID_hypCGeometricBase,GetOuterUnknown(),
			m_spUnknownVertices.iid(),(void**)&m_spUnknownVertices);
		CGlobalFactory::CreateInstance(CLSID_hypCContainer,0,
			m_spSideContKey.iid(),(void**)&m_spSideContKey);			
	} catch(hyp_ker::CComException& hr) {
		return hyp_ker::com_false;
	}
	return hyp_ker::com_ok;
}


void CGeometricObject::FinalRelease(void)
{
	CUnknown::FinalRelease();
}


//IGeometricObject
IGeometricSide* CGeometricObject::CreateSide(const t_label& Label,const t_label_enum& LabelEnum)
{
	CreateSide(Label);
	t_spGeometricBase p_side=m_spSideContKey->GetContent(Label);
	IGeometricVertex* p_vertex;
	t_spGeometricBase spVerticesBase=m_spUnknownVertices;
	for(int i=0;i<LabelEnum.size();i++) {
		p_vertex=spVerticesBase->GetVertex(LabelEnum[i]);
		p_side->AddVertex(p_vertex,i);
	}

	//on convertit
	t_spGeometricSide spSide=p_side;
	return spSide;
}

IGeometricSide* CGeometricObject::CreateSide(const t_label& Label)
{
	t_spGeometricSide spSide;
	CGlobalFactory::CreateInstance(CLSID_hypCGeometricSide,0,spSide.iid(),(void**)&spSide);
	m_spSideContKey->Add(spSide,Label);
	
	return spSide;
}

IGeometricVertex* CGeometricObject::CreateVertex(const t_label& Label,const t_real& X,const t_real& Y,const t_real& Z)
{
	CreateVertex(Label);
	t_spGeometricBase spVerticesBase=m_spUnknownVertices;
	IGeometricVertex* p_vertex=spVerticesBase->GetVertex(Label);
	p_vertex->SetCoord(X,0);
	p_vertex->SetCoord(Y,1);
	p_vertex->SetCoord(Z,2);

	return p_vertex;
}

IGeometricVertex* CGeometricObject::CreateVertex(const t_label& Label)
{
	t_spGeometricVertex spVertex;
	CGlobalFactory::CreateInstance(*m_pCLSID_Vertex,0,spVertex.iid(),(void**)&spVertex);
	
	t_spGeometricBase spVerticesBase=m_spUnknownVertices;
	spVerticesBase->AddVertex(spVertex,Label);
	return spVertex;
}


//		
void CGeometricObject::AddSide(IGeometricSide* pObject,const t_label& Label)
{
	m_spSideContKey->Add(pObject,Label);
}

void CGeometricObject::DeleteSide(const t_label& Label)
{
	m_spSideContKey->Remove(Label);
}

IContainerKey* CGeometricObject::GetContainerOfSides()
{
	return m_spSideContKey;
}

t_size CGeometricObject::GetNumberOfSides()
{
	t_spContainer spCont=m_spSideContKey;
	return spCont->GetSize();
}

IGeometricSide* CGeometricObject::GetSide(const t_label& Label)
{
	t_spGeometricSide spSide=m_spSideContKey->GetContent(Label);
	return spSide;
}

/*
///////////
//cf. p.598
void CGeometricObject::InitializeD3DRMMesh(LPDIRECT3DRMMESHBUILDER2 pMeshBuilder)
{
	t_spGeometricObject spGeometricObjectThis=this;
	hypfac::CGeometryData data(spGeometricObjectThis);
	hypfac::CGeometryData::t_vertices_enum vertices_enum=data.GetVerticesEnum();

	
	D3DVECTOR* vertices=new D3DVECTOR[vertices_enum.size()];
	int offset=0;
	for(int i=0;i<vertices_enum.size();i++) {
		vertices[offset].x=vertices_enum[i].coord[0];
		vertices[offset].y=vertices_enum[i].coord[1];
		vertices[offset].z=vertices_enum[i].coord[2];
		
		offset++;
	}

	hypfac::CGeometryData::t_sides_enum sides_enum=data.GetSidesEnum();
	
	t_size nSideLabels=0;
	for(i=0;i<sides_enum.size();i++) {
		nSideLabels+=sides_enum[i].vertices_global.size();
	}

	DWORD* face_data=new DWORD[sides_enum.size()+nSideLabels];
	offset=0;
	for(i=0;sides_enum.size();i++) {
		face_data[offset]=sides_enum[i].vertices_global.size();
		offset++;
		for(int j=0;j<sides_enum[i].vertices_global.size();j++) {
			face_data[offset]=sides_enum[i].vertices_global[j];
			offset++;
		}
	}

	HRESULT hr=pMeshBuilder->AddFaces(vertices_enum.size(),vertices,
		0,NULL,
		face_data,
		NULL);
	
	delete[] vertices;
	delete[] face_data;	
	
	if(FAILED(hr)) hyp_THROW_COM(hyp_ker::com_false);
}
*/

hyp_NAMESPACE_END(data)
hyp_NAMESPACE_END_HYPFEM