/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 14/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#ifndef _hyp_data_side_h_
#define _hyp_data_side_h_

#ifndef hypfem_INSIDE_INTERN
#error Inclusion not allowed (include hyp.fem.intern.h instead)
#endif

/*!
\ingroup Ephy_FEM_Data
\HypDefaultDescripComponent IGeometricSide.
\HypCLSID CLSID_hypCGeometricSide {68038B80-1DD5-11d5-A094-0080AD97951B}
\HypBeginInterf hyp.data.side.cpp
\HypEndInterf

\HypImplDetails
The only diference between the CGeometricSide component and the CGeometricBase component
is that the CGeometricSide sets to true the visual property of the vertex during the insertion procedure.
See AddVertex method.
*/
class CGeometricSide :
#ifdef hypfem_IDENTIFY_CONSTRUCTION_ON
protected CIDClass,
#endif
public IGeometricBase,
public IGeometricSide,
public hyp_ker::CUnknown
{
public:
	/*! \HypCommonFonct*/
	CGeometricSide(hyp_ker::IUnknown*);
	virtual ~CGeometricSide();
	hyp_DECLARE_IUNKNOWN
	virtual hyp_ker::com_result NondelegatingQueryInterface(const hyp_ker::hypIID&,void**);
	static hyp_ker::com_result CreateInstance(hyp_ker::IUnknown* ,hyp_ker::CUnknown**);
	hyp_ker::com_result Init(void);	
	void FinalRelease(void);
	/*!@}*/

	/*! \name IGeometricBase Interface
	@{
	*/
	virtual t_size GetNumberOfVertices() {
		return GetGeometricBase()->GetNumberOfVertices(); }
	virtual IGeometricVertex* GetVertex(const t_label& Label) {
		return GetGeometricBase()->GetVertex(Label);
	}
	virtual IGeometricVertex* GetVertex(const t_real& X,const t_real& Y,const t_real& Z) {
		return GetGeometricBase()->GetVertex(X,Y,Z);
	}
	virtual t_label GetLabel(IGeometricVertex* pObject) {
		return GetGeometricBase()->GetLabel(pObject);
	}
	virtual void AddVertex(IGeometricVertex*,const t_label&);
	virtual void DeleteVertex(const t_label& Label) {
		GetGeometricBase()->DeleteVertex(Label);
	}
	virtual IContainerKey* GetContainerOfVertices() {
		return GetGeometricBase()->GetContainerOfVertices();
	}
	virtual void SetFilter(const hyp_ker::hypIID* pIID) {
		GetGeometricBase()->SetFilter(pIID);
	}
	virtual void ClearFilter() {
		GetGeometricBase()->ClearFilter();
	}
	/*!@}*/

private:
	inline IGeometricBase* GetGeometricBase(void) {
		t_spGeometricBase spVerticesBase=m_spUnknownBase;
		return spVerticesBase;
	}
	hyp_ker::IPtrUnknown m_spUnknownBase;
};

#endif