/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(fem)

CFEOElements::CFEOElements(hyp_ker::IUnknown* pUnknownOuter,PTR_INIT_MATRIX init_matrix) :
CUnknown(pUnknownOuter),
m_InitMatrix(init_matrix)
{
	hypfem_TRACE_CONSTRUCTION(("CFEOElements"));
}

CFEOElements::~CFEOElements()
{
	hypfem_TRACE_DESTRUCTOR(("~CFEOElement"));
}


hyp_ker::com_result CFEOElements::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIFEOElement) {
		return FinishQI(static_cast<IFEOElement*>(this),ppv);
	} else if(iid==IID_hypIGeometricBase ||
		iid==IID_hypIContainer ||
		iid==IID_hypIContainerKey ||
		iid==IID_hypIContainerIterator) {
		return m_spUnknownContNodes->QueryInterface(iid,ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG

hyp_ker::com_result CFEOElements::Init(void)
{
	hyp_ker::com_result hr=hyp_ker::com_ok;
	try{
		CGlobalFactory::CreateInstance(CLSID_hypCGeometricBase,GetOuterUnknown(),
			m_spUnknownContNodes.iid(),(void**)&m_spUnknownContNodes);	
		GetGeometricBase()->SetFilter(&IID_hypIFEOBoundaryConditions);
	} catch(hyp_ker::CComException hr) {
		hr=hyp_ker::com_false;
	}
	return hr;
}

void CFEOElements::FinalRelease(void)
{
	CUnknown::FinalRelease();
}

/*!
Creates an brick element. Refrenced in Ansys by Brick42.
\todo insert image
At each call of the SetKe, the component fills the matrix with the following procedure :

\include hyp.fem.ke.cpp

Note that the implmentation checks if the nodes of the element are well-ordered.
*/
hyp_ker::com_result CFEOElements::CreateInstanceBrick42(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CFEOElements(pUnknownOuter,CFEOElements::InitMatrixForBrick42);
	return hyp_ker::com_ok;
}


const IFEOMaterial* CFEOElements::GetMaterial() const
{
	return m_pMaterial;
}

void CFEOElements::SetMaterial(IFEOMaterial* pObject)
{
	m_pMaterial=pObject;
}

void CFEOElements::SetKe(IMatrix* pObject)
{
	m_InitMatrix(this,pObject);
}


hyp_NAMESPACE_END(fem)
hyp_NAMESPACE_END_HYPFEM