/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#include"hyp.fem.intern.h"

using namespace hyp::hypfem;

IFactoriesLibrary* CGlobalFactory::m_Inst=0;

#ifdef hypfem_IDENTIFY_CONSTRUCTION_ON
int CIDClass::ms_NbGlobalInstances=0;
#endif

#include"hyp.data.object.cpp"




hyp_EXPORT_DLL void hyp_CALL_CONV InitLibrary(const std::vector<std::string>& Param,
												   long Reason)
{
	switch(Reason)
	{
	case hyp_ker::dll_attach : {
		hyp_ker::CFactoryData FactoryDataArray[]={
			hyp_ker::CFactoryData(&CLSID_hypCContainer,common::CContainer::CreateInstance,"Container"),
			hyp_ker::CFactoryData(&CLSID_hypCFactoriesLibrary,common::CFactoriesLibrary::CreateInstance,"FactoriesLibrary"),

			hyp_ker::CFactoryData(&CLSID_hypCMathMatrix,math::CMatrix::CreateInstance,"MathMatrix"),
			hyp_ker::CFactoryData(&CLSID_hypCMathVector,math::CVector::CreateInstance,"MathVector"),			

			hyp_ker::CFactoryData(&CLSID_hypCGeometricVertex,data::CGeometricVertex::CreateInstance,"GeometricVertex"),
			hyp_ker::CFactoryData(&CLSID_hypCGeometricBase,data::CGeometricBase::CreateInstance,"GeometricBase"),
			hyp_ker::CFactoryData(&CLSID_hypCGeometricSide,data::CGeometricSide::CreateInstance,"GeometricSide"),
			hyp_ker::CFactoryData(&CLSID_hypCGeometricObject,data::CGeometricObject::CreateInstanceForGeometry,"GeometricObject"),
			hyp_ker::CFactoryData(&CLSID_hypCGeometricObjectForFiniteElement,data::CGeometricObject::CreateInstanceForFiniteElement,"GeometricObjectForFiniteElement"),

			hyp_ker::CFactoryData(&CLSID_hypCDefaultMaterial,fem::CFEOMaterials::CreateInstanceDefault,"DefaultMaterial"),
			hyp_ker::CFactoryData(&CLSID_hypCSteelMaterial,fem::CFEOMaterials::CreateInstanceSteel,"SteelMaterial"),
			hyp_ker::CFactoryData(&CLSID_hypCBrick42Element,fem::CFEOElements::CreateInstanceBrick42,"Brick42"),
			hyp_ker::CFactoryData(&CLSID_hypCFEONode,fem::CFEONode::CreateInstance,"FEONode"),
			hyp_ker::CFactoryData(&CLSID_hypCFEOMesh,fem::CFEOMesh::CreateInstance,"FEOMesh")				
		};			

		int cFactoryDataEntries= sizeof(FactoryDataArray) / sizeof(hyp_ker::CFactoryData);
		hyp_ker::CFactory::InitStaticMembers(FactoryDataArray,cFactoryDataEntries,*Param.begin());
			
		hypfem_TRACE( ("Appel de InitLibrary %s",Param.begin()->c_str()) );
		break;
	}
	//DOXYTAG
		
	case hyp_ker::dll_detach : {
		if(CGlobalFactory::IsInit()) {
			CGlobalFactory::Inst()->Release();
		}

		//bilan memory
		hyp_ker::cerr<<"\nBILAN->"<<CIDClass::GetNbGlobalInstances()<<"\n";
		break;
	}
		
	default: {
		hyp_ASSERT(0);
	}

	}
}

hyp_DECLARE_EXPORTED_FUNCTIONS