/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#ifndef _hyp_fem_intern_h_
#define _hyp_fem_intern_h_

/*! \file hyp.fem.intern.h
Includes necessary headers to manipulate the Hyperion Components. If you don't want using
the notion of component, include only hyp.kernel.core.h file.
*/

/*!
Avoids wrong file inclusion.
*/
#define hypfem_INSIDE_INTERN

#define hyp_CATCH_ITOR  catch(hyp_ker::CComError error) {				\
							if(hyp_ker::hyp_FAILED(error.GetError())) {	\
								throw error;							\
							}											\
						}



#include"hyp.fem.extern.h"

//////////////
#define hypfem_USE_GLOBAL_FACTORY
#define hypfem_TRACE_ON
#define hypfem_TRACE_DESTRUCTOR_ON
#define hypfem_IDENTIFY_CONSTRUCTION_ON

#ifdef hypfem_TRACE_ON
#define hypfem_TRACE( args )						\
		hyp_TRACE_BEGIN;							\
		hyp_TRACE_DO( ("Trace from fem server") );	\
		hyp_TRACE_DO( args );						\
		hyp_TRACE_END;
#else
#define hypfem_TRACE( args )
#endif

#ifdef hypfem_TRACE_DESTRUCTOR_ON
#define hypfem_TRACE_DESTRUCTOR( args ) hypfem_TRACE( args )
#endif

#ifdef hypfem_IDENTIFY_CONSTRUCTION_ON
class CIDClass {
protected:
	CIDClass() {
		hyp_ker::CTime _t;
		_t.SetCurrent();
		std::stringstream str_stream;
		str_stream<<_t;
		m_ID=str_stream.str();
		ms_NbGlobalInstances++;
	}
	virtual ~CIDClass() {
		ms_NbGlobalInstances--;
		if(ms_NbGlobalInstances==0) hyp_ker::clog<<"!!!No Memory Leak!!!";
	}
	std::string m_ID;
	static int ms_NbGlobalInstances;
public:
	static int GetNbGlobalInstances(void) { return ms_NbGlobalInstances; }
};
#define hypfem_TRACE_CONSTRUCTION( args )	\
	hyp_TRACE_BEGIN;						\
	hyp_TRACE_DO( args );					\
{ std::stringstream _stream;_stream<<"-"<<this;m_ID+=_stream.str();	}\
	hyp_TRACE_DO( ("Creation %s",m_ID.c_str()) );	\
	hyp_TRACE_END;							
#else
#define hypfem_TRACE_CONSTRUCTION( args )
#endif

//////////


#include"extern_library/newmat.h"


hyp_NAMESPACE_BEGIN_HYPFEM

//DOXYTAG
#ifndef hypfem_DOXYGEN

namespace common {
#include"hyp.com.container.h"
#include"hyp.com.factorieslibrary.h"
}

namespace math {
typedef NEWMAT::Matrix t_LibraryMatrix;
typedef NEWMAT::ColumnVector t_LibraryVector;
#include"hyp.math.matrix.h"
#include"hyp.math.vector.h"
}

namespace data {
#include"hyp.data.vertex.h"
#include"hyp.data.base.h"
#include"hyp.data.side.h"
#include"hyp.data.object.h"
}

namespace fem {
#include"hyp.fem.node.h"
#include"hyp.fem.element.h"
#include"hyp.fem.mesh.h"
#include"hyp.fem.materials.h"
}

#endif

class CGlobalFactory
{
	CGlobalFactory();
	static IFactoriesLibrary* m_Inst;

public:
	static IFactoriesLibrary* Inst(void) {
		if(m_Inst==0) {
			hyp_ker::CreateInstance(CLSID_hypCFactoriesLibrary,0,
				hyp_ker::INPROC_SERVER,
				IID_hypIFactoriesLibrary,
				(void**)&m_Inst);
		}
		return m_Inst;
	}
	static bool IsInit(void) { return m_Inst ? true:false; }

	static void CreateInstance(const hyp_ker::hypCLSID& clsid,hyp_ker::IUnknown* pI,const hyp_ker::hypIID& iid,void** ppv) 
	{
#ifdef hypfem_USE_GLOBAL_FACTORY
		Inst()->CreateInstance(clsid,pI,hyp_ker::INPROC_SERVER,iid,ppv);
#else
		hyp_ker::hypCreateInstance(clsid,pI,hyp_ker::INPROC_SERVER,iid,ppv);
#endif
	}

};

hyp_NAMESPACE_END_HYPFEM


#endif	//!defined(_hyp_data_intern_h_)