/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(fem)


CFEOMaterials::CFEOMaterials(hyp_ker::IUnknown* pUnknownOuter,
							 t_real young,
							 t_real poisson) :
hyp_ker::CUnknown(pUnknownOuter),
m_Young(young),
m_Poisson(poisson)
{
	hypfem_TRACE_CONSTRUCTION(("CFEOMaterials"));
}

CFEOMaterials::~CFEOMaterials()
{
	hypfem_TRACE_DESTRUCTOR(("~CFEOMaterials"));
}


hyp_ker::com_result CFEOMaterials::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIFEOMaterial) {
		return FinishQI(static_cast<IFEOMaterial*>(this),ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG

hyp_ker::com_result CFEOMaterials::Init(void)
{
	return hyp_ker::com_ok;
}

void CFEOMaterials::FinalRelease(void)
{
	CUnknown::FinalRelease();
}


//voir poly RDM (debut)
/*!
Creates the default material.
- E = 100000.0f
- Nu = 0.20f

\internal
*/
hyp_ker::com_result CFEOMaterials::CreateInstanceDefault(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CFEOMaterials(pUnknownOuter,100000.0f,0.20f);
	return hyp_ker::com_ok;
}

/*!
Creates steel material. Also designated by Steel E24.
- E = 210e6f
- Nu = 0.28f

\internal
*/
hyp_ker::com_result CFEOMaterials::CreateInstanceSteel(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	//acier doux E24
	*ppNewComponent=new CFEOMaterials(pUnknownOuter,210e6f,0.28f);
	return hyp_ker::com_ok;
}

//IFEOMaterial
void CFEOMaterials::SetYoung(const t_real& Scalar)
{
	m_Young=Scalar;
}

t_real CFEOMaterials::GetYoung() const
{
	return m_Young;
}

void CFEOMaterials::SetPoisson(const t_real& Scalar)
{
	m_Poisson=Scalar;
}

t_real CFEOMaterials::GetPoisson() const
{
	return m_Poisson;
}

hyp_NAMESPACE_END(fem)
hyp_NAMESPACE_END_HYPFEM
