/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(fem)


CFEOMesh::CFEOMesh(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter)
{
	hypfem_TRACE_CONSTRUCTION(("CFEOMesh"));

}

CFEOMesh::~CFEOMesh()
{
	hypfem_TRACE_DESTRUCTOR(("~CFEOMesh"));
}


hyp_ker::com_result CFEOMesh::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIFEOMesh) {
		return FinishQI(static_cast<IFEOMesh*>(this),ppv);
	} else if(iid==IID_hypIGeometricBase 
		|| iid==IID_hypIGeometricObject
		|| iid==IID_hypIContainer
		|| iid==IID_hypIContainerKey
		|| iid==IID_hypIContainerIterator/*
		|| iid==IID_hypID3DRMMeshObject*/) {
		return m_spUnknownGeometricObject->QueryInterface(iid,ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG

hyp_ker::com_result CFEOMesh::Init(void)
{
	hyp_ker::com_result hr=hyp_ker::com_ok;
	try{
		CGlobalFactory::CreateInstance(CLSID_hypCGeometricObjectForFiniteElement,GetOuterUnknown(),
			m_spUnknownGeometricObject.iid(),(void**)&m_spUnknownGeometricObject);
		CGlobalFactory::CreateInstance(CLSID_hypCContainer,0,
			m_spElementsContKey.iid(),(void**)&m_spElementsContKey);
		CGlobalFactory::CreateInstance(CLSID_hypCDefaultMaterial,0,
			m_spDefaultMaterial.iid(),(void**)&m_spDefaultMaterial);
		CGlobalFactory::CreateInstance(CLSID_hypCMathMatrix,0,
			m_spGlobalMatrix.iid(),(void**)&m_spGlobalMatrix);

	} catch(hyp_ker::CComException hr) {
		hr=hyp_ker::com_false;
	}

	return hr;
}

void CFEOMesh::FinalRelease(void)
{
	CUnknown::FinalRelease();
}

hyp_ker::com_result CFEOMesh::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CFEOMesh(pUnknownOuter);
	return hyp_ker::com_ok;
}

/////////////////
/////////////////
//CMaskData
t_size CFEOMesh::CMaskData::ms_Dim=0;

CFEOMesh::CMaskData::CMaskData(int dim) :
m_Data(dim,0.0f),
m_Mask(dim,true)
{
	SetMask(dim);
}

void CFEOMesh::CMaskData::SetMask(const t_size& dim)
{
	ms_Dim=dim;
	m_Mask.resize(ms_Dim,true);
	m_Data.resize(ms_Dim,0.0f);
}

void CFEOMesh::CMaskData::SetMask(const t_mask_array& mask)
{
	hyp_ASSERT(mask.size()==m_Mask.size());	

	m_Mask=mask;
	m_Data.resize(GetMaskDim(),0.0f);
}

void CFEOMesh::CMaskData::MergeMask(const t_mask_array& mask)
{
	hyp_ASSERT(mask.size()==m_Mask.size());

	for(int i=0;i<m_Mask.size();i++) {
		if(mask[i]==false) m_Mask[i]=mask[i];
	}
	m_Data.resize(GetMaskDim(),0.0f);
}

void CFEOMesh::CMaskData::SetData(const t_real_array& a)
{
	hyp_ASSERT(a.size()==m_Data.size());

	m_Data=a;
}

t_size CFEOMesh::CMaskData::GetMaskDim(void) const
{
	t_size nbr_true=0;
	for(int i=0;i<m_Mask.size();i++) {
		if(m_Mask[i]) nbr_true++;
	}
	return nbr_true;
}

void CFEOMesh::CMaskData::SetGlobalData(const t_real_array& a)
{
	hyp_ASSERT(a.size()==m_Mask.size());

	int j=0;
	for(int i=0;i<m_Mask.size();i++) {
		if(m_Mask[i]) {
			m_Data[j]=a[i];
			j++;
		}
	}	
}

void CFEOMesh::CMaskData::GetGlobalData(t_real_array& a) const
{
	hyp_ASSERT(a.size()==m_Mask.size());

	int j=0;
	for(int i=0;i<m_Mask.size();i++) {
		if(m_Mask[i]) {
			a[i]=m_Data[j];
			j++;
		} else {
			a[i]=0;
		}
	}
}

void CFEOMesh::CMaskData::SetData(const IMatrix* p_matrix)
{
	CHECK_MATRIX_VALIDITY(p_matrix);

	for(int i=0;i<m_Data.size();i++) {
		m_Data[i]=p_matrix->GetValue(i,0);
	}
}

void CFEOMesh::CMaskData::ClearData(t_real r)
{
	for(int i=0;i<m_Data.size();i++) {
		m_Data[i]=r;		
	}
}

void CFEOMesh::CMaskData::SetVector(IMatrix* p_matrix) const
{
	CHECK_MATRIX_VALIDITY(p_matrix);
	
	for(int i=0;i<m_Data.size();i++) {
		p_matrix->SetValue(i,0,m_Data[i]);
	}
}

t_size CFEOMesh::CMaskData::GetPos(const t_label& label,const int& dim) const
{
	int pos=(label*3)+dim;
	hyp_ASSERT(pos<m_Mask.size());

	//renvoie -1 si pas dans le masque
	if(m_Mask[pos]==false) return -1;

	//combien de true dans le masque  pos
	int nbr=0;
	for(int i=0;i<pos;i++) {
		if(m_Mask[i]==true) {
			nbr++;
		}
	}
	return nbr;	
}


void CFEOMesh::SetDefaultMaterial(IFEOMaterial* pObject)
{
	hyp_ASSERT(pObject);
	m_spDefaultMaterial=pObject;
}

const IFEOMaterial* CFEOMesh::GetDefaultMaterial() const
{
	return m_spDefaultMaterial;
}

/////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////
//Assemblage
t_label CFEOMesh::GetNodeLOCE(IGeometricVertex* p_node,t_size dim)
{
	hyp_ASSERT(dim>=0 && dim<3);
	return (3*GetGeometricBase()->GetLabel(p_node))+dim;
}

t_label_enum CFEOMesh::GetElementLOCE(IFEOElement* p_element)
{
	t_label_enum LOCE;

	t_spContainerIterator spItor_Element=p_element;
	try {
		t_spGeometricVertex spVertex=spItor_Element->SetBeginSelection();
		while(1) {
			for(int i=0;i<3;i++) {
				LOCE.push_back(GetNodeLOCE(spVertex,i));
			}
			spVertex=spItor_Element->IncrSelection();
		}
	} hyp_CATCH_COM_ITOR
	return LOCE;
}

void CFEOMesh::ShowLOCE(std::ostream& stream,const t_label_enum& loce,const std::string& doc)
{
	stream<<doc<<"\n";
	for(int i=0;i<loce.size();i++) {
		stream<<i<<"->"<<loce[i]<<"\n";
	}
}

t_size CFEOMesh::GetDOF(void)
{
	return 3*GetGeometricBase()->GetNumberOfVertices();
}


void CFEOMesh::ConstructGlobalMatrix()
{
	m_spGlobalMatrix->SetDim(GetDOF(),GetDOF(),0);

	t_spContainerIterator spElementsItor=m_spElementsContKey;
	//matrice lmentaire de rigidit
	t_spMathMatrix spMatrix;
	spMatrix.CreateInstance(CLSID_hypCMathMatrix,0);
	//table LOCE
	t_label_enum LOCE;

	try{
		t_spFEOElement spElement=spElementsItor->SetBeginSelection();
		while(1) {
			spElement->SetKe(spMatrix);
			LOCE=GetElementLOCE(spElement);
			//FIXTRACE
			//ShowLOCE(hyp_ker::hyp_clog,LOCE,"element loce");
			for(int i=0;i<spMatrix->GetNbRow();i++) {
				for(int j=0;j<spMatrix->GetNbColumn();j++) {
					m_spGlobalMatrix->IncrValue(LOCE[i],LOCE[j],spMatrix->GetValue(i,j));
				}
			}		
			spElement=spElementsItor->IncrSelection();
		}
	} hyp_CATCH_COM_ITOR

	m_Displacement.SetMask(GetDOF());	
	m_RealTimeForce.SetMask(GetDOF());
}

IMatrix* CFEOMesh::K() const
{
	/*
	if(!m_spGlobalMatrix) {
		hyp_THROW_COM2(hyp_ker::com_false, ("K has not been constructed.") );
	}
	*/
	return m_spGlobalMatrix;	
}


//false qd c'est bloqu, true sinon
t_mask_array CFEOMesh::GetDOFMask()
{
	t_mask_array mask_array(GetDOF(),false);

	t_label label=0;
	
	t_spFEOBoundaryConditions spBConditions;
	t_spGeometricVertex spVertex;
	t_spContainerIterator spItor=m_spUnknownGeometricObject;
	try{
		spBConditions=spItor->SetBeginSelection();
		while(1) {
			spVertex=spBConditions;
			for(int i=0;i<3;i++) {
				mask_array[GetNodeLOCE(spVertex,i)]=!(spBConditions->IsFixed(i));
			}
			spBConditions=spItor->IncrSelection();
		}
	} hyp_CATCH_COM_ITOR

	return mask_array;
}

t_mask_array CFEOMesh::GetVisibleMask()
{
	t_mask_array visible_array(GetDOF(),false);

	t_spGeometricVertex spVertex;
	t_spContainerIterator spItor=m_spUnknownGeometricObject;
	try{
		int i=0;
		spVertex=spItor->SetBeginSelection();
		while(1) {
			if(spVertex->IsVisible()) {
				visible_array[i]=true;
				visible_array[i+1]=true;
				visible_array[i+2]=true;
			}
			i+=3;
			spVertex=spItor->IncrSelection();
		}
	} hyp_CATCH_COM_ITOR

	return visible_array;
}

void CFEOMesh::ConstructBoundaryMatrix()
{
	t_mask_array mask=GetDOFMask();
	m_spGlobalMatrix->EraseRowsAndColumns(mask,mask);

	m_Displacement.SetMask(mask);	
	m_RealTimeForce.SetMask(mask);
}

void CFEOMesh::InverseBoundaryMatrix()
{
	m_spGlobalMatrix->Invert();	
}


//doit tre appel quand on load une matrix
void CFEOMesh::InitMasks()
{
	t_mask_array column_mask=GetDOFMask();
	t_mask_array row_mask=GetVisibleMask();

	//on merge pour obtenir row
	for(int i=0;i<row_mask.size();i++) {
		if(column_mask[i]==false)
			row_mask[i]=false;
	}
	
	m_Displacement.SetMask(GetDOF());
	m_RealTimeForce.SetMask(GetDOF());
	m_Displacement.SetMask(row_mask);	
	m_RealTimeForce.SetMask(column_mask);
}

void CFEOMesh::ConstructVisibleMatrix()
{
	hyp_ASSERT( (m_Displacement.Mask().size()==m_RealTimeForce.Mask().size()) &&
		(m_Displacement.Mask().size()==GetDOF()) );

	//on simplfie la matrice en enlevant les points qui ne sont pas visibles
	t_mask_array visible_mask=GetVisibleMask();

	//on merge les deux masks
	t_mask_array matrix_mask(m_Displacement.GetMaskDim(),true);
	int j=0;
	for(int i=0;i<m_Displacement.Mask().size();i++) {
		if(m_Displacement.Mask(i)) {
			if(visible_mask[i]==false) {
				matrix_mask[j]=false;
			}
			j++;
		}
	}


	m_spGlobalMatrix->EraseRowsAndColumns(matrix_mask,t_mask_array(matrix_mask.size(),true));
	m_Displacement.MergeMask(visible_mask);

#if 1
	hyp_ker::cerr<<"*******\n";
	for(i=0;i<m_Displacement.Data().size();i++) {
		hyp_ker::cerr<<m_Displacement.Data(i)<<"\n";
	}

#endif

}


/////////////////
/////////////////
//Mask
void CFEOMesh::SetRenderMask(const t_mask_array& Mask)
{
	m_Displacement.SetMask(Mask);
}

t_mask_array CFEOMesh::RenderMask()
{
	return m_Displacement.Mask();
}

t_real_array CFEOMesh::Displacement()
{
	return m_Displacement.Data();
}

t_real_array CFEOMesh::GetGlobalDisplacement()
{
	t_real_array a(GetDOF(),0.0f);
	m_Displacement.GetGlobalData(a);
	return a;
}

void CFEOMesh::SetRealTimeMask(const t_mask_array& Mask)
{
	m_RealTimeForce.SetMask(Mask);
}

t_mask_array CFEOMesh::RealTimeMask()
{
	return m_RealTimeForce.Mask();
}

t_real_array CFEOMesh::RealTimeForce()
{
	//hyp_ASSERT(0);
	return m_RealTimeForce.Data();
}

t_real_array CFEOMesh::GetGlobalRealTimeForce()
{
	t_real_array a(GetDOF(),0.0f);
	m_RealTimeForce.GetGlobalData(a);
	return a;
}

void CFEOMesh::SetGlobalRealTimeForce(const t_real_array& Array)
{
	m_RealTimeForce.SetGlobalData(Array);
}

void CFEOMesh::ClearRealTimeForce()
{
	//FIXTODO
	t_real value=0.0f;
	m_RealTimeForce.ClearData(0.0f);
}


//////////////
//////////////
void CFEOMesh::PreRender()
{
	//perform some initialisations

}

void CFEOMesh::Render()
{
	//creation de F
	t_spMathMatrix spF;
	spF.CreateInstance(CLSID_hypCMathVector,0);
	spF->SetDim(m_RealTimeForce.GetMaskDim(),1,0);	
	m_RealTimeForce.SetVector(spF);

	
	t_spMathMatrix spU;
	spU.CreateInstance(CLSID_hypCMathVector,0);
	spU->SetDim(m_Displacement.GetMaskDim(),1,0);
			
	m_spGlobalMatrix->Multiply(spF,spU);

	m_Displacement.SetData(spU);

//FIXREMOVE
#if 0
	hyp_ker::hyp_clog<<"result**********************\n";
	for(int i=0;i<spF->GetNbRow();i++) {
		hyp_ker::hyp_clog<<spF->GetValue(i,0)<<"-----\n";
		hyp_ker::hyp_clog<<spU->GetValue(i,0)<<"\n";
	}
#endif
	
}

void CFEOMesh::PostRender()
{
	//on bouge les points
	//FIXTODO : construire un cache!!!!
	IGeometricVertex* p_vertex=0;
	for(int i=0;i<GetDOF()/3;i++) {
		p_vertex=GetGeometricBase()->GetVertex(i);
		int pos=m_Displacement.GetPos(i,0);
		for(int j=0;j<3;j++) {
			t_real coord=p_vertex->GetCoord(j);
			if(pos>=0) {
				p_vertex->SetCoord(m_Displacement.Data()[pos+j]+coord,j);
			}
		}
	}
}

//////////////
//////////////
//ajout de force
//on recupre la valeur par RealTimeForce, GetGlobalRealTimeForce, etc.
void CFEOMesh::SetForce(const t_label& Label,const t_real& FX,const t_real& FY,const t_real& FZ)
{
	hyp_ASSERT(Label*3+2<GetDOF());

	int offset=0;

	offset=m_RealTimeForce.GetPos(Label,0);
	if(offset>=0) m_RealTimeForce.Data(offset)+=FX;

	offset=m_RealTimeForce.GetPos(Label,1);
	if(offset>=0) m_RealTimeForce.Data(offset)+=FY;

	offset=m_RealTimeForce.GetPos(Label,2);
	if(offset>=0) m_RealTimeForce.Data(offset)+=FZ;
}


//
IGeometricBase* CFEOMesh::CreateElement(const t_label& Label,const t_label_enum& LabelEnum)
{
	t_spGeometricBase spBase_Element;
	t_spFEOElement spElement;
	t_spGeometricBase sp_IGeometricBase=m_spUnknownGeometricObject;

	//FIXTODO : differents types d'elements
	spElement.CreateInstance(CLSID_hypCBrick42Element,0);
	m_spElementsContKey->Add(spElement,Label);

	IGeometricVertex* p_vertex=0;
	for(int i=0;i<LabelEnum.size();i++) {
		p_vertex=sp_IGeometricBase->GetVertex(LabelEnum[i]);
		spBase_Element=spElement;
		spBase_Element->AddVertex(p_vertex,i);
		spElement->SetMaterial(m_spDefaultMaterial);
	}
	return spBase_Element;
}

IContainerKey* CFEOMesh::GetContainerOfElements()
{
	return m_spElementsContKey;
}

t_size CFEOMesh::GetNumberOfElements()
{
	t_spContainer spContainer=m_spElementsContKey;
	return spContainer->GetSize();
}

IGeometricBase* CFEOMesh::GetElement(const t_label& Label)
{
	t_spGeometricBase spGeometricBase=m_spElementsContKey->GetContent(Label);
	return spGeometricBase;
}


//conditions aux limites
void CFEOMesh::FixDOF(const t_real& X,const t_real& Y,const t_real& Z,bool FilterX,bool FilterY,bool FilterZ,bool DOFX,bool DOFY,bool DOFZ)
{
	//on cherche les noeuds qui correpond aux critres
	t_spContainerIterator spItor=GetGeometricBase()->GetContainerOfVertices();
	t_spFEOBoundaryConditions spBConditions;
	t_spGeometricVertex spVertex;
	try{
		spVertex=spItor->SetBeginSelection();
		while(1) {
			if( ( (!FilterX) || (FilterX && spVertex->GetCoord(0)==X) ) &&
				( (!FilterY) || (FilterY && spVertex->GetCoord(1)==Y) ) &&
				( (!FilterZ) || (FilterZ && spVertex->GetCoord(2)==Z) ) ) {
				spBConditions=spVertex;
				if(DOFX) spBConditions->FixDOF(0);
				if(DOFY) spBConditions->FixDOF(1);
				if(DOFZ) spBConditions->FixDOF(2);
			}
			spVertex=spItor->IncrSelection();
		}
	} hyp_CATCH_COM_ITOR
}

void CFEOMesh::SetForce(const t_real& X,const t_real& Y,const t_real& Z,
						bool FilterX,bool FilterY,bool FilterZ,
						const t_real& FX,const t_real& FY,const t_real& FZ)
{
	//on cherche les noeuds qui correpond aux critres
	t_spContainerIterator spItor=GetGeometricBase()->GetContainerOfVertices();
	t_spGeometricVertex spVertex;
	try{
		spVertex=spItor->SetBeginSelection();
		while(1) {
			if( ( (!FilterX) || (FilterX && spVertex->GetCoord(0)==X) ) &&
				( (!FilterY) || (FilterY && spVertex->GetCoord(1)==Y) ) &&
				( (!FilterZ) || (FilterZ && spVertex->GetCoord(2)==Z) ) ) {
				SetForce(GetGeometricBase()->GetLabel(spVertex),FX,FY,FZ);
			}
			spVertex=spItor->IncrSelection();
		}
	} hyp_CATCH_COM_ITOR

#if 0
	hyp_ker::hyp_cerr<<"force***\n";
	for(int i=0;i<m_RealTimeForce.Data().size();i++) {
		hyp_ker::hyp_cerr<<m_RealTimeForce.Data(i)<<"\n";
	}
#endif
}

void CFEOMesh::ClearAllData()
{
	t_spContainer spCont;

	//efface nodes
	t_spGeometricBase spBase=m_spUnknownGeometricObject;
	spCont=spBase->GetContainerOfVertices();
	spCont->RemoveAll();
	
	//efface sides
	t_spGeometricObject spObject=m_spUnknownGeometricObject;
	spCont=spObject->GetContainerOfSides();
	spCont->RemoveAll();

	//efface elements
	spCont=m_spElementsContKey;
	spCont->RemoveAll();

	//reduit global matrix
	m_spGlobalMatrix->SetDim(0,0,0);

	//FIXTODO : effacer d'autres objects intermdiaire

}

bool CFEOMesh::IsSideFixed(const t_label& SideLabel,const t_size& CoordinateLabel)
{
	t_spContainerKey spContKey=GetGeometricObject()->GetContainerOfSides();
	t_spGeometricBase spSide=spContKey->GetContent(SideLabel);
	t_spContainerIterator spContItor=spSide->GetContainerOfVertices();

	t_spFEOBoundaryConditions spBConditions;

	try{
		spBConditions=spContItor->SetBeginSelection();
		while(1) {
			if(spBConditions->IsFixed(CoordinateLabel)==false) return false;
			
			spBConditions=spContItor->IncrSelection();
		}
	} hyp_CATCH_COM_ITOR

	return true;
}



hyp_NAMESPACE_END(fem)
hyp_NAMESPACE_END_HYPFEM