/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_fem_mesh_h_
#define _hyp_fem_mesh_h_

#ifndef hypfem_INSIDE_INTERN
#error Inclusion not allowed (include hyp.fem.intern.h instead)
#endif


/*!
\ingroup Ephy_FEM_FEO
\HypDefaultDescripComponent IFEOMesh.
\HypCLSID CLSID_hypCFEOMesh {2CC70D26-AB21-11d4-A098-0080AD97951B}
\HypBeginInterf hyp.fem.mesh.cpp
\HypEndInterf

\HypImplDetails
*/
class CFEOMesh :
#ifdef hypfem_IDENTIFY_CONSTRUCTION_ON
protected CIDClass,
#endif
public IFEOMesh,
public hyp_ker::CUnknown
{
public:
	/*! \HypCommonFonct*/
	CFEOMesh(hyp_ker::IUnknown*);
	virtual ~CFEOMesh();
	hyp_DECLARE_IUNKNOWN
	virtual hyp_ker::com_result NondelegatingQueryInterface(const hyp_ker::hypIID&,void**);
	static hyp_ker::com_result CreateInstance(hyp_ker::IUnknown* ,hyp_ker::CUnknown** );
	hyp_ker::com_result Init(void);	
	void FinalRelease(void);
	/*!@}*/

	/*! \name IFEOMesh Interface
	@{
	*/
	virtual void SetDefaultMaterial(IFEOMaterial*);
	virtual const IFEOMaterial* GetDefaultMaterial() const;
	virtual void ConstructGlobalMatrix();
	virtual IMatrix* K() const;
	virtual void ConstructBoundaryMatrix();
	virtual void InverseBoundaryMatrix();
	virtual void ConstructVisibleMatrix();
	virtual void InitMasks();

	virtual void SetRenderMask(const t_mask_array&);
	virtual t_mask_array RenderMask();
	virtual t_real_array Displacement();
	virtual t_real_array GetGlobalDisplacement();
	
	virtual void SetRealTimeMask(const t_mask_array&);
	virtual t_mask_array RealTimeMask();
	virtual t_real_array RealTimeForce();
	virtual t_real_array GetGlobalRealTimeForce();
	virtual void SetGlobalRealTimeForce(const t_real_array&);
	virtual void ClearRealTimeForce();

	virtual void PreRender();
	virtual void Render();
	virtual void PostRender();

	virtual IGeometricBase* CreateElement(const t_label&,const t_label_enum&);
	virtual IContainerKey* GetContainerOfElements();
	virtual t_size GetNumberOfElements();
	virtual IGeometricBase* GetElement(const t_label&);

	virtual void SetForce(const t_label&,const t_real&,const t_real&,const t_real&);
	virtual void SetForce(const t_real&,const t_real&,const t_real&,bool,bool,bool,const t_real&,const t_real&,const t_real&);
	virtual void FixDOF(const t_real&,const t_real&,const t_real&,bool,bool,bool,bool,bool,bool);

	virtual void ClearAllData();

	virtual bool IsSideFixed(const t_label&,const t_size&);
	/*!@}*/
	

private:
	//pour l'assemblage
	t_label_enum GetElementLOCE(IFEOElement*);
	t_label GetNodeLOCE(IGeometricVertex*,t_size);
	void ShowLOCE(std::ostream&,const t_label_enum&,const std::string&);

	//le nombre de ddl, cad taille matrix
	t_size GetDOF(void);
	
	//le masque en function des ddl bloqus
	t_mask_array GetDOFMask(void);
	t_mask_array GetVisibleMask(void);

private:
	//aggregation
	hyp_ker::IPtrUnknown m_spUnknownGeometricObject;
	inline IGeometricObject* GetGeometricObject(void) {
		t_spGeometricObject spGeometricObject=m_spUnknownGeometricObject;
		return spGeometricObject;
	}
	inline IGeometricBase* GetGeometricBase(void) {
		t_spGeometricBase spGeometricBase=m_spUnknownGeometricObject;
		return spGeometricBase;
	}

	//liste des lments
	t_spContainerKey m_spElementsContKey;

	//material par defaut
	t_spFEOMaterial m_spDefaultMaterial;

	t_spMathMatrix m_spGlobalMatrix;

	class CMaskData{
	public:
		CMaskData(int =0);
		CMaskData(const CMaskData&);	//not implemented

		void SetMask(const t_size&);
		void SetMask(const t_mask_array&);
		void MergeMask(const t_mask_array&);
		void SetData(const t_real_array&);
		
		inline const t_mask_array& Mask(void) const { return m_Mask; }
		inline const t_real_array& Data(void) const { return m_Data; }
		
		inline t_real& Data(t_size i) { hyp_ASSERT( i<m_Data.size() ); return m_Data[i]; }
		inline bool& Mask(t_size i) { hyp_ASSERT( i<m_Mask.size() ); return m_Mask[i]; }
		inline const t_real& Data(t_size i) const { hyp_ASSERT( i<m_Data.size() ); return m_Data[i]; }
		inline const bool& Mask(t_size i) const { hyp_ASSERT( i<m_Mask.size() ); return m_Mask[i]; }

		t_size GetMaskDim(void) const;

		void SetGlobalData(const t_real_array&);
		void GetGlobalData(t_real_array&) const;
		void SetVector(IMatrix*) const;
		void SetData(const IMatrix*);
		void ClearData(t_real);

		//donne le label absolu d'une force,d'un node(ex 0->0/1->0/2->0/3->1..)
		t_size GetPos(const& t_label,const int&) const;

	private:
		inline void CHECK_MATRIX_VALIDITY(const IMatrix* p_matrix) const {
			hyp_ASSERT(p_matrix);
			hyp_ASSERT( (p_matrix->GetNbColumn()==1) && 
				(p_matrix->GetNbRow()==m_Data.size()) );
		}
		static t_size ms_Dim;
		t_mask_array m_Mask;
		t_real_array m_Data;
	};	

	CMaskData m_RealTimeForce;
	CMaskData m_Displacement;
};


#endif