/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 26/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.fem.intern.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(fem)

CFEONode::CFEONode(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter),
m_DOFFixed(3)
{
	hypfem_TRACE_CONSTRUCTION(("CFEONode"));
}

CFEONode::~CFEONode()
{
	hypfem_TRACE_DESTRUCTOR(("~CFEONode"));
}


hyp_ker::com_result CFEONode::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIFEOBoundaryConditions) {
		return FinishQI(static_cast<IFEOBoundaryConditions*>(this),ppv);
	} else if(iid==IID_hypIGeometricVertex) {
		return m_spUnknownVertex->QueryInterface(iid,ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG

hyp_ker::com_result CFEONode::Init(void)
{
	hyp_ker::com_result hr=hyp_ker::com_ok;
	try {
		CGlobalFactory::CreateInstance(CLSID_hypCGeometricVertex,GetOuterUnknown(),
			m_spUnknownVertex.iid(),(void**)&m_spUnknownVertex);
	} catch(hyp_ker::CComException hr) {
		hr=hyp_ker::com_false;
	}
	return hr;
}

void CFEONode::FinalRelease(void)
{
	CUnknown::FinalRelease();
}

hyp_ker::com_result CFEONode::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CFEONode(pUnknownOuter);
	return hyp_ker::com_ok;
}

void CFEONode::FixDOF(const t_size& Label)
{
	hyp_ASSERT(Label>=0 && Label<3);
	m_DOFFixed[Label]=true;
}

void CFEONode::UnFixDOF(const t_size& Label)
{
	hyp_ASSERT(Label>=0 && Label<3);
	m_DOFFixed[Label]=false;
}

void CFEONode::ClearDOF()
{
	m_DOFFixed[0]=false;
	m_DOFFixed[1]=false;
	m_DOFFixed[2]=false;
}

bool CFEONode::IsFixed(const t_size& Label) const
{
	hyp_ASSERT(Label>=0 && Label<3);
	return m_DOFFixed[Label];
}

hyp_NAMESPACE_END(fem)
hyp_NAMESPACE_END_HYPFEM
