/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 22/03/01 P.Naves
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.fem.intern.h"
#include"extern_library/newmat.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(math)


//FIXTODO
//methodes locales
t_size GetDimMask(const t_mask_array&);



CMatrix::CMatrix(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter)
{
	hypfem_TRACE_CONSTRUCTION(("CMatrix"));
}

CMatrix::~CMatrix()
{
	hypfem_TRACE_DESTRUCTOR(("~CMatrix"));
}


hyp_ker::com_result CMatrix::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIMathMatrix) {
		return FinishQI(static_cast<IMatrix*>(this),ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG


hyp_ker::com_result CMatrix::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CMatrix(pUnknownOuter);
	return hyp_ker::com_ok;
}


hyp_ker::com_result CMatrix::Init(void)
{
	// l'objet CMatrix cr est de dimension 1X1
	// son seul lment est initialis  0.
	SetDim(1,1,0.0);
	
	return hyp_ker::com_ok;
}

void CMatrix::FinalRelease(void)
{
	CUnknown::FinalRelease();
}




void CMatrix::Add(const IMatrix* B) 
// this+=Matrix_b
{
	t_LibraryMatrix provMatrice;

	try {
		_CopyMatrixFromComposant(B, provMatrice);
		Matrix_biblio+=provMatrice;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Add error: incompatible dimensions"));
	}
}

void CMatrix::Invert() 
// this=inverse de this
{
	t_LibraryMatrix provMatrice;
	try {
		provMatrice=Matrix_biblio.i();
		_CopyMatrixFromImpl(provMatrice,this);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Invert error: singular matrix"));
	}
}

void CMatrix::Transpose(IMatrix* B)
// Matrix_b=transpose de this
{
	t_LibraryMatrix provMatrice;
	try {
		provMatrice.ReSize(this->GetNbColumn(),this->GetNbRow());
		provMatrice=Matrix_biblio.t();
		_CopyMatrixFromImpl(provMatrice,B);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Transpose error"));
	}
}


void CMatrix::Substract(const IMatrix* B) 
// this-=Matrix_b
{
	t_LibraryMatrix provMatrice;

	try {
		_CopyMatrixFromComposant(B,provMatrice);
		Matrix_biblio-=provMatrice;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Substract error: incompatible dimensions"));
	}

}

void CMatrix::Multiply(const t_real& Scalar) 
// this=scalar_b * Matrix_b
{
	try {
	Matrix_biblio=Scalar * Matrix_biblio;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false, ("Multiply by a scalar error"));
	}
}

t_size CMatrix::GetNbColumn() const 
// Renvoie le nb de colonnes de this
{
	try {
		return Matrix_biblio.Ncols();
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false, ("Error in GetNbColumn"));
	}

}

t_size CMatrix::GetNbRow() const 
// Renvoie le nb de lignes de this
{
	try {
		return Matrix_biblio.Nrows();
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in GetNbRow"));
	}

}

void CMatrix::SetMatrix(const t_real& Scalar) 
// Initialise tous les lments de
// this  scalar_b
{												
	try {
		Matrix_biblio=Scalar;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in SetMatrix"));
	}

}

t_real CMatrix::GetValue(const t_size& X, const t_size& Y) const 
// Renvoie l'lment (x,y) de this
{
	try {
		return Matrix_biblio(X+1,Y+1);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("GetValue error: index out of range"));
	}

}

void CMatrix::SetValue(const t_size& X,const t_size& Y,const t_real& Scalar) 
// Met l'lment (x,y) de this  la valeur scalar_b
{
	try {
		Matrix_biblio(X+1,Y+1)=Scalar;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("SetValue error: index out of range"));
	}

}

void CMatrix::SetDim(const t_size& Column, const t_size& Row, const t_real& Scalar) 
// Initialise les dimensions de this  (x,y)
// tous les lments sont initialiss  init
{																			
	try {
		Matrix_biblio.ReSize(Column,Row);
		Matrix_biblio=Scalar;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in SetDim"));
	}
}

void CMatrix::IncrValue(const t_size& X,const t_size& Y,const t_real& Scalar)
// L'lment (x,y) de this est incrment de scalar_b
{
	try {
		Matrix_biblio(X+1,Y+1)+=Scalar;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("IncrValue error: index out of range"));
	}

}


void CMatrix::Extract(const t_size& MinX, const t_size& MaxX, const t_size& MinY, const t_size& MaxY, IMatrix *B)
// On extrait de this une sous-matrice de dimension ( x_max - x_min , y_max - y_min )
// La sous matrice est stocke dans Matrix_b
// Matrix_b->GetValue(0,0)=this->GetValue(x_min,y_min)
{
	t_LibraryMatrix provMatrice;

	try {
		_CopyMatrixFromComposant(this,provMatrice);
		provMatrice=provMatrice.SubMatrix(MinX+1,MaxX+1,MinY+1,MaxY+1);
		_CopyMatrixFromImpl(provMatrice,B);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Extract error"));
	}
}

void CMatrix::SetMatrix(IMatrix *B)
// Initialise this  la matrice Matrix_b
{
	t_LibraryMatrix provMatrice;
	try {
		_CopyMatrixFromComposant(B,provMatrice);
		_CopyMatrixFromImpl(provMatrice,this);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in SetMatrix"));
	}

}


void CMatrix::Multiply(const IMatrix* B, IMatrix* C)
// Matrix_c = this * Matrix_b
// !!! Le rsultat est stock dans Matrix_c
{
	t_LibraryMatrix provMatrice;
	t_LibraryVector provVecteur;
	

	try {
		if (B->GetNbColumn() == 1) // On multiplie this par un vecteur
		{
			_CopyVectorFromComposant(B,provVecteur);
			provVecteur=(this->Matrix_biblio)*provVecteur;
			_CopyVectorFromImpl(provVecteur,C);
		}
		else // On multiplie this par une autre matrice
		{
			_CopyMatrixFromComposant(B,provMatrice);
			provMatrice=(this->Matrix_biblio)*provMatrice;
			_CopyMatrixFromImpl(provMatrice,C);
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Multiply by a IMatrix error"));
	}
}

//ajout 29/03/01 Tycho
bool CMatrix::IsSymmetrical()
{
	t_LibraryMatrix TMatrix=Matrix_biblio;
	TMatrix.t();
	TMatrix-=Matrix_biblio;

	return TMatrix.IsZero();
}

void CMatrix::EraseRow(const t_size& Y)
{	
	if(!(Y<GetNbRow()-1)) hyp_THROW_COM(hyp_ker::com_false);
	t_LibraryMatrix MatrixProv(GetNbRow()-1,GetNbColumn());
	for(int i=1;i<=MatrixProv.Nrows();i++) {
		for(int j=1;j<=MatrixProv.Ncols();j++) {
			if(i<=Y) {
				MatrixProv(i,j)=Matrix_biblio(i,j);
			} else {
				MatrixProv(i,j)=Matrix_biblio(i+1,j);
			}
		}
	}
	Matrix_biblio.ReSize(GetNbRow()-1,GetNbColumn());
	Matrix_biblio=MatrixProv;

}

void CMatrix::EraseColumn(const t_size& X)
{
	if(!(X<GetNbColumn()-1)) hyp_THROW_COM(hyp_ker::com_false);
	t_LibraryMatrix MatrixProv(GetNbRow(),GetNbColumn()-1);
	for(int i=1;i<=MatrixProv.Nrows();i++) {
		for(int j=1;j<=MatrixProv.Ncols();j++) {
			if(j<=X) {
				MatrixProv(i,j)=Matrix_biblio(i,j);
			} else {
				MatrixProv(i,j)=Matrix_biblio(i,j+1);
			}
		}
	}
	Matrix_biblio.ReSize(GetNbRow(),GetNbColumn()-1);
	Matrix_biblio=MatrixProv;
}

/*!
\todo inversion param
*/
void CMatrix::EraseRowsAndColumns(const t_mask_array& RowMask, const t_mask_array& ColumnMask)
{
	if(RowMask.size()!=Matrix_biblio.Nrows()) hyp_THROW_COM(hyp_ker::com_false);
	if(ColumnMask.size()!=Matrix_biblio.Ncols()) hyp_THROW_COM(hyp_ker::com_false);

	t_LibraryMatrix MatrixProv(GetDimMask(RowMask),GetDimMask(ColumnMask));

	int i_prov=0;
	int j_prov=0;
	for(int i=0;i<Matrix_biblio.Nrows();i++) {
		if(RowMask[i]==true) {
			for(int j=0;j<Matrix_biblio.Ncols();j++) {
				if(ColumnMask[j]==true) {
					MatrixProv(i_prov+1,j_prov+1)=Matrix_biblio(i+1,j+1);
					j_prov++;
				}
			}
			j_prov=0;
			i_prov++;
		}		
	}
	Matrix_biblio.ReSize(MatrixProv.Nrows(),MatrixProv.Ncols());
	Matrix_biblio=MatrixProv;
}


t_size GetDimMask(const t_mask_array& mask)
{
	int j=0;
	for(int i=0;i<mask.size();i++) {
		if(mask[i]==true) {
			j++;
		}
	}

	return j;
}




//////////////////////
// METHODES PRIVEES///
//////////////////////

void CMatrix::_CopyMatrixFromComposant(const IMatrix* pIMatriceACopier,t_LibraryMatrix& DestMatrice) const
// Remplie DestMatrice tel que DestMatrice==pIMatriceACopier
// Permet de crer un objet de type Matrix (bibliothque matricielle dont nous nous servons)
//  partir d'un objet IMatrix
{
	int col,row;	//contient les dimensions du composant  copier
	int i,j; //pour les boucles

	try {
		row=pIMatriceACopier->GetNbRow();
		col=pIMatriceACopier->GetNbColumn();
	
		DestMatrice.ReSize(row,col);

		for(i=0;i<row;i++) {
			for(j=0;j<col;j++) {
				DestMatrice(i+1,j+1)=pIMatriceACopier->GetValue(i,j);
			}
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Copy error"));
	}
}



void CMatrix::_CopyMatrixFromImpl(const t_LibraryMatrix& SourceMatrice,IMatrix* pIMatriceDest)
// Remplie pIMatriceDest tel que SourceMatrice=pIMatriceDest
// type Matrix -> type IMatrix
{
	int col=SourceMatrice.Ncols();
	int row=SourceMatrice.Nrows();
	int i,j;

	try {
		pIMatriceDest->SetDim(row,col);

		for(i=0;i<row;i++) {
			for(j=0;j<col;j++) {
				pIMatriceDest->SetValue(i,j,SourceMatrice(i+1,j+1));
			}
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Copy error"));
	}

}


void CMatrix::_CopyVectorFromComposant(const IMatrix* pIVecteurACopier,t_LibraryVector& DestVector) const
// Remplie DestVector tel que DestVector==pIVecteurACopier
// type IMatrix -> type ColumnVector
{
	int row;	//contient les dimensions du composant  copier
	int i; //pour la boucle

	try {
		row=pIVecteurACopier->GetNbRow();
	
		DestVector.ReSize(row);

		for(i=0;i<row;i++) {
			DestVector(i+1)=pIVecteurACopier->GetValue(i,0);
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Copy error"));
	}

}



void CMatrix::_CopyVectorFromImpl(const t_LibraryVector& SourceVecteur,IMatrix* pIVecteurDest)
// Remplie pIVecteurDest tel que SourceVecteur=pIVecteurDest
// type ColumnVector -> type IMatrix
{
	try {
		int row=SourceVecteur.Nrows();
		int i;

		pIVecteurDest->SetDim(row,1);
		for(i=0;i<row;i++) {
				pIVecteurDest->SetValue(i,0,SourceVecteur(i+1));
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Copy error"));
	}

}

hyp_NAMESPACE_END(math)
hyp_NAMESPACE_END_HYPFEM


