/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Creation 22/03/01 P.Naves
 Add 29/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#ifndef _hyp_math_matrix_h_
#define _hyp_math_matrix_h_

#ifndef hypfem_INSIDE_INTERN
#error Inclusion not allowed (include hyp.fem.intern.h instead)
#endif

/*!
\ingroup Ephy_FEM_Math
\HypDefaultDescripComponent IMatrix
\HypCLSID CLSID_hypCMathMatrix {BF8B3F61-1F01-11d5-A095-0080AD97951B}
\HypBeginInterf hyp.math.matrix.cpp
\HypEndInterf

\HypImplDetails
CMatrix has been crated to manage specially the huge matrices which characterize the bodies,
i.e the rigidity matrix.
\HypMatLibrary
*/
class CMatrix :
#ifdef hypfem_IDENTIFY_CONSTRUCTION_ON
protected CIDClass,
#endif
public IMatrix,
public hyp_ker::CUnknown
{
public:
	/*! \HypCommonFonct*/
	CMatrix(hyp_ker::IUnknown*);
	virtual ~CMatrix();
	hyp_DECLARE_IUNKNOWN
	virtual hyp_ker::com_result NondelegatingQueryInterface(const hyp_ker::hypIID&,void**);
	static hyp_ker::com_result CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown**);
	hyp_ker::com_result Init(void);	
	void FinalRelease(void);
	/*!@}*/

	/*! \name IMatrix Implementation
	@{
	*/
	virtual void Transpose(IMatrix*);
	virtual void SetMatrix(IMatrix*);
	virtual void Multiply (const IMatrix* ,IMatrix*);
	virtual void Extract (const t_size& ,const t_size& ,const t_size& , const t_size& ,IMatrix* );
	virtual void IncrValue(const t_size& ,const t_size& ,const t_real& );
	virtual void SetDim(const t_size& , const t_size& , const t_real& );
	virtual void SetValue(const t_size& , const t_size& , const t_real& );
	virtual t_real GetValue(const t_size& ,const t_size& ) const;
	virtual void SetMatrix(const t_real& );
	virtual t_size GetNbRow() const;
	virtual t_size GetNbColumn() const;
	virtual void Multiply(const t_real& );
	virtual void Substract(const IMatrix* );
	virtual void Invert();
	virtual void Add(const IMatrix* );
	virtual bool IsSymmetrical();
	virtual void EraseRow(const t_size& );
	virtual void EraseColumn(const t_size& );
	virtual void EraseRowsAndColumns(const t_mask_array&, const t_mask_array&);
	/*!@}*/

private:
	t_LibraryMatrix Matrix_biblio;
	void _CopyMatrixFromComposant( const IMatrix* ,t_LibraryMatrix& ) const;
	void _CopyMatrixFromImpl(const t_LibraryMatrix& ,IMatrix* );
	void _CopyVectorFromComposant(const IMatrix* ,t_LibraryVector& ) const;
	void _CopyVectorFromImpl(const t_LibraryVector& ,IMatrix* );

};


#endif