/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 22/03/01 P.Naves
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.fem.intern.h"
#include"extern_library/newmat.h"

hyp_NAMESPACE_BEGIN_HYPFEM
hyp_NAMESPACE_BEGIN(math)


CVector::CVector(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter)
{
	hypfem_TRACE_CONSTRUCTION(("CVector"));
}

CVector::~CVector()
{
	hypfem_TRACE_DESTRUCTOR(("~CVector"));
}


hyp_ker::com_result CVector::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIMathMatrix) {
		return FinishQI(static_cast<IMatrix*>(this),ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG


hyp_ker::com_result CVector::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CVector(pUnknownOuter);
	return hyp_ker::com_ok;
}


hyp_ker::com_result CVector::Init(void)
{
	// l'objet CVector cr est de dimension 1
	// son seul lment est initialis  0.
	SetDim(1,1,0.0);
	
	return hyp_ker::com_ok;
}

void CVector::FinalRelease(void)
{
	CUnknown::FinalRelease();
}





void CVector::Add(const IMatrix* B) 
//this+=t_LibraryMatrix_b
{
	t_LibraryVector provVector;

	try {
		_CopyVectorFromComposant(B,provVector);
		this->Vector_biblio+=provVector;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Add error: incompatible dimensions"));
	}

}

void CVector::Substract(const IMatrix* B) 
//this-=t_LibraryMatrix_b
{
	t_LibraryVector provVector;

	try {
		_CopyVectorFromComposant(B,provVector);
		this->Vector_biblio-=provVector;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Substract error: incompatible dimensions"));
	}

}

void CVector::Multiply(const t_real& Scalar) 
// this = scalar_b * t_LibraryMatrix_b
{
	try {
		this->Vector_biblio=Scalar * this->Vector_biblio;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false, ("Multiply by a scalar error"));
	}
}

void CVector::Multiply(const IMatrix* B, IMatrix* C)
// t_LibraryMatrix_c = this * t_LibraryMatrix_b
// !!! Le rsultat est stock dans t_LibraryMatrix_c
// Multiplication d'un vecteur colonne (this) par un vecteur ligne (t_LibraryMatrix_c)

{
	t_LibraryMatrix provMatrice, provResultat;

	try {
		_CopyMatrixFromComposant(B,provMatrice);
		provResultat=(this->Vector_biblio)*provMatrice;
		_CopyMatrixFromImpl(provResultat,C);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Multiply by a It_LibraryMatrix error"));
	}
}

void CVector::Invert()
// Impossibilit d'inverser un vecteur: on lance une exception
{
	hyp_THROW_COM2(hyp_ker::com_false,("Cannot invert a vector"));
}

void CVector::Transpose(IMatrix* B)
// t_LibraryMatrix_b=transpose de this
// t_LibraryMatrix_b est en fait un vecteur ligne
// Ici il sera simplement de type t_LibraryMatrix
// avec pour dimension (1,this->GetNbRow())
{
	t_LibraryMatrix provMatrice;
	try {
		provMatrice.ReSize(1,this->GetNbColumn());
		provMatrice=Vector_biblio.t();
		_CopyMatrixFromImpl(provMatrice,B);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Transpose error"));
	}
}




t_size CVector::GetNbColumn() const
// Renvoie le nb de colonnes de this  savoir 1
{
    return 1;                     
}

t_size CVector::GetNbRow() const
// Renvoie le nb de lignes de this
{
	try {
		return this->Vector_biblio.Nrows();
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in GetNbRow"));
	}
}

void CVector::SetMatrix(const t_real& Scalar)
// Initialise tous les lments de
// this  scalar_b
{
	try {
		this->Vector_biblio=Scalar;
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in SetMatrix"));
	}
}


void CVector::SetMatrix(IMatrix *B)
// Initialise this  la matrice t_LibraryMatrix_b
{
	try {
		t_LibraryVector provVector;
		_CopyVectorFromComposant(B,provVector);
		_CopyVectorFromImpl(provVector,this);
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in SetMatrix"));
	}
}


t_real CVector::GetValue(const t_size& X, const t_size& Y) const
// Renvoie l'lment (x) de this
// y doit tre  0

{
	try {
		if (Y!=0) {
			hyp_THROW_COM(hyp_ker::com_false);
		} else {
			return this->Vector_biblio(X+1);
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("GetValue error: index out of range"));
	}

}

void CVector::SetValue(const t_size& X,const t_size& Y,const t_real& Scalar)
// Met l'lment (x) de this  la valeur scalar_b
// y doit tre  0
{
	try {
		if (Y!=0) {
			hyp_THROW_COM(hyp_ker::com_false);
		} else {
			this->Vector_biblio(X+1)=Scalar;
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("SetValue error: index out of range"));
	}

}

void CVector::SetDim(const t_size& Column, const t_size& Row, const t_real& Scalar)
// Initialise les dimensions de this  x
// y doit tre  1
// tous les lments sont initialiss  init

{
	try {
		if (Row!=1) {
			hyp_THROW_COM(hyp_ker::com_false);
		} else {
			this->Vector_biblio.ReSize(Column);
			this->Vector_biblio=Scalar;
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Error in SetDim"));
	}
}

void CVector::IncrValue(const t_size& X,const t_size& Y,const t_real& Scalar)
// L'lment (x) de this est incrment de scalar_b
//y doit tre  0
{
	try {
		if (Y!=0) {
			hyp_THROW_COM(hyp_ker::com_false);
		} else {
			this->Vector_biblio(X+1) += Scalar;
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("IncrValue error: index out of range"));
	}
}


void CVector::Extract(const t_size& MinX, const t_size& MaxX, const t_size& MinY, const t_size& MaxY, IMatrix *B)
{
	t_LibraryVector provVecteur;

	try {
		if ( (MinY!=0) || (MaxY!=0)) {
			hyp_THROW_COM(hyp_ker::com_false);
		} else {
			_CopyVectorFromComposant(this,provVecteur);
			provVecteur=provVecteur.Rows(MinX+1,MaxX+1);
			_CopyVectorFromImpl(provVecteur,B);
		}
	} catch(...) {
		hyp_THROW_COM2(hyp_ker::com_false,("Extract error"));
	}
}




//////////////////////
// METHODES PRIVEES///
//////////////////////

void CVector::_CopyMatrixFromComposant(const IMatrix* pIMatriceACopier,t_LibraryMatrix& DestMatrice) const
// Remplie DestMatrice tel que DestMatrice==pIMatriceACopier
// Permet de crer un objet de type t_LibraryMatrix (bibliothque matricielle dont nous nous servons)
//  partir d'un objet It_LibraryMatrix
{
	int col,row;	//contient les dimensions du composant  copier
	int i,j; //pour les boucles

	try {
		row=pIMatriceACopier->GetNbRow();
		col=pIMatriceACopier->GetNbColumn();
	
		DestMatrice.ReSize(row,col);

		for(i=0;i<row;i++) {
			for(j=0;j<col;j++) {
				DestMatrice(i+1,j+1)=pIMatriceACopier->GetValue(i,j);
			}
		}
	} catch(...) {
		hyp_THROW_COM(hyp_ker::com_false);
	}
}



void CVector::_CopyMatrixFromImpl(const t_LibraryMatrix& SourceMatrice,IMatrix* pIMatriceDest)
// Remplie pIMatriceDest tel que SourceMatrice=pIMatriceDest
// type t_LibraryMatrix -> type It_LibraryMatrix
{
	int col=SourceMatrice.Ncols();
	int row=SourceMatrice.Nrows();
	int i,j;

	try {
		pIMatriceDest->SetDim(row,col);

		for(i=0;i<row;i++) {
			for(j=0;j<col;j++) {
				pIMatriceDest->SetValue(i,j,SourceMatrice(i+1,j+1));
			}
		}
	} catch(...) {
		hyp_THROW_COM(hyp_ker::com_false);
	}

}


void CVector::_CopyVectorFromComposant(const IMatrix* pIVecteurACopier,t_LibraryVector& DestVector) const
// Remplie DestVector tel que DestVector==pIVecteurACopier
// type It_LibraryMatrix -> type t_LibraryVector
{
	int row;	//contient les dimensions du composant  copier
	int i; //pour la boucle

	try {
		row=pIVecteurACopier->GetNbRow();
	
		DestVector.ReSize(row);

		for(i=0;i<row;i++) {
			DestVector(i+1)=pIVecteurACopier->GetValue(i,0);
		}
	} catch(...) {
		hyp_THROW_COM(hyp_ker::com_false);
	}

}



void CVector::_CopyVectorFromImpl(const t_LibraryVector& SourceVecteur,IMatrix* pIVecteurDest)
// Remplie pIVecteurDest tel que SourceVecteur=pIVecteurDest
// type t_LibraryVector -> type It_LibraryMatrix
{
	try {
		int row=SourceVecteur.Nrows();
		int i;

		pIVecteurDest->SetDim(row,1);
		for(i=0;i<row;i++) {
				pIVecteurDest->SetValue(i,0,SourceVecteur(i+1));
		}
	} catch(...) {
		hyp_THROW_COM(hyp_ker::com_false);
	}

}

hyp_NAMESPACE_END(math)
hyp_NAMESPACE_END_HYPFEM



