/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 22/03/01 P.Naves
 Add 29/03/01 Pierre Rebours Miscellenious functions of the IMatrix interface
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_math_vector_h_
#define _hyp_math_vector_h_


#ifndef hypfem_INSIDE_INTERN
#error Inclusion not allowed (include hyp.fem.intern.h instead)
#endif

/*!
\ingroup Ephy_FEM_Math
CVector is a implementation of the IMatrix which is used in specific cases.
\HypCLSID CLSID_hypCMathVector {BF8B3F62-1F01-11d5-A095-0080AD97951B}
\HypBeginInterf hyp.math.vector.cpp
\HypEndInterf

\HypImplDetails
CVector has been crated to manage specially the huge unidimentionnal matrices (i.e vectors)
which characterize the loading conditions or the displacemnts of the nodes.
\HypMatLibrary
*/
class CVector :
#ifdef hypfem_IDENTIFY_CONSTRUCTION_ON
protected CIDClass,
#endif
public IMatrix,
public hyp_ker::CUnknown
{
public:
	/*! \HypCommonFonct*/
	CVector(hyp_ker::IUnknown*);
	virtual ~CVector();
	hyp_DECLARE_IUNKNOWN
	virtual hyp_ker::com_result NondelegatingQueryInterface(const hyp_ker::hypIID&,void**);
	static hyp_ker::com_result CreateInstance(hyp_ker::IUnknown* ,hyp_ker::CUnknown**);
	hyp_ker::com_result Init();	
	void FinalRelease();
	/*!@}*/

	/*! \name IMatrix Implementation
	@{
	*/
	virtual void Transpose(IMatrix* );
	virtual void SetMatrix(IMatrix* );
	virtual void Multiply (const IMatrix* , IMatrix* );
	virtual void Extract (const t_size& ,const t_size& ,const t_size& , const t_size& , IMatrix* );
	virtual void IncrValue(const t_size& ,const t_size& ,const t_real& );
	virtual void SetDim(const t_size& , const t_size& , const t_real& );
	virtual void SetValue(const t_size& , const t_size& , const t_real& );
	virtual t_real GetValue(const t_size& ,const t_size& ) const;
	virtual void SetMatrix(const t_real& );
	virtual t_size GetNbRow() const;
	virtual t_size GetNbColumn() const;
	virtual void Multiply(const t_real& );
	virtual void Substract(const IMatrix* );
	virtual void Invert();
	virtual void Add(const IMatrix* );
	virtual bool IsSymmetrical(void) { return false; }
	virtual void EraseRow(const t_size& ) { hyp_ASSERT(0); }
	virtual void EraseColumn(const t_size& ) { hyp_ASSERT(0); }
	virtual void EraseRowsAndColumns(const t_mask_array&, const t_mask_array&) { hyp_ASSERT(0); }
	/*!@}*/

private:
	t_LibraryVector Vector_biblio;
	void _CopyMatrixFromComposant(const IMatrix* pIMatriceACopier,t_LibraryMatrix& DestMatrice) const;
	void _CopyMatrixFromImpl(const t_LibraryMatrix& SourceMatrice,IMatrix* pIMatriceDest);
	void _CopyVectorFromComposant(const IMatrix* pIMatriceACopier,t_LibraryVector& DestMatrice) const;
	void _CopyVectorFromImpl(const t_LibraryVector& SourceMatrice,IMatrix* pIMatriceDest);



};

#endif
