/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 02/05/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#ifndef _hyp_file_chunkx_h_
#define _hyp_file_chunkx_h_

class CChunkBase
{
public:
	char* m_pChunk;
	DWORD m_Size;
	int m_Offset;

	CChunkBase() : m_pChunk(0),m_Size(0),m_Offset(0) {}

	inline DWORD Size(void) const {
		return m_Size;
	}
	inline void* Chunk(void) const {
		return static_cast<void*>(m_pChunk);
	}
};

class CChunkMemory :
public CChunkBase
{
	
public:
	CChunkMemory(const DWORD& size) {
		m_Size=size;
		m_pChunk=new char[size];
		memset(m_pChunk,0,size);
		m_Offset=0;
	}
	~CChunkMemory() {
		delete[] m_pChunk;
	}


	template<class T>
	void InsertData(const T* p_data) {
		hyp_ASSERT(m_Offset+sizeof(T)<=m_Size);
		memcpy(&m_pChunk[m_Offset],p_data,sizeof(T));
		m_Offset+=sizeof(T);
	}

	template<class T>
	void InsertBases(const T& bases) {
		DWORD dw_value=bases.size();
		InsertData(&dw_value);
	
		for(int i=0;i<bases.size();i++) {
			dw_value=bases[i].label;
			InsertData(&dw_value);
			dw_value=bases[i].vertices_global.size();
			InsertData(&dw_value);
			
			for(int j=0;j<bases[i].vertices_global.size();j++) {
				dw_value=bases[i].vertices_global[j];
				InsertData(&dw_value);
			}
			
			for(j=0;j<bases[i].vertices_local.size();j++) {
				dw_value=bases[i].vertices_local[j];
				InsertData(&dw_value);
			}
		}
	}
};

class CChunkMemoryRead :
public CChunkBase
{
public:
	CChunkMemoryRead(LPDIRECTXFILEDATA p_data)
	{
		p_data->GetData(NULL,&m_Size,(void**)&m_pChunk);
	}

	template<class T>
	void Extract(T* p_data) {
		hyp_ASSERT(m_Offset+sizeof(T)<=m_Size);
		memcpy(p_data,&m_pChunk[m_Offset],sizeof(T));
		m_Offset+=sizeof(T);
	}
};

	

#endif