/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 02/05/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#ifndef _hyp_file_formatstd_h_
#define _hyp_file_formatstd_h_

class CValue
{
protected:
	std::string m_Value;	
	template<class T> std::string ConvertValue(const T& value) {
		std::stringstream buff;
		buff<<value;
		return buff.str();
	}
	
public:
	template<class T> 
	CValue(const T& value) {
		SetValue(value);
	}
	CValue() {}
	CValue(const CValue& value) { m_Value=value.m_Value; }

	template<class T> void SetValue(const T& value) {
		m_Value.erase();
		m_Value=ConvertValue(value);
	}

	template<class T> void InsertNewValue(const T& value) {
		if(!m_Value.empty()) m_Value+="-";
		m_Value+=ConvertValue(value);
	}

	inline std::string& Value(void) { return m_Value; }
	inline const std::string& Value(void) const { return m_Value; }
};


class CDataToShow
{
protected:
	int m_Size;
public:
	CDataToShow(const int& size) : m_Size(size) {}
	CDataToShow() : m_Size(0) {}
	CDataToShow(const CDataToShow& data) { m_Size=data.Size(); }
	inline int& Size(void) { return m_Size; }
	inline const int& Size(void) const { return m_Size; }
	virtual void write(std::ostream&) const =0;
};

class CBox :
public CValue,
public CDataToShow
{
public:
	template<class T> 
	CBox(const T& value,int size) : CValue(value), CDataToShow(size) {}
	CBox(const CBox& box) {
		Value()=box.Value();
		Size()=box.Size();
	}
	
	virtual void write(std::ostream& stream) const;

	static const int ms_ColumnSize_Bool;
	static const int ms_ColumnSize_Label;
	static const int ms_ColumnSize_Real;
	static const int ms_ColumnSize_List;
};

class CColumn :
public CDataToShow
{
public:
	static const int ms_PosColumn;
	std::vector<CBox> m_BoxEnum;
	CColumn() : CDataToShow(ms_PosColumn) {}
	CColumn(const CColumn& col) { Size()=col.Size(); }

	virtual void write(std::ostream& stream) const;
};

class CColumnHead :
public CColumn
{
public:
	virtual void write(std::ostream& stream) const;
};

class CTable :
public CDataToShow,
public CValue
{
public:
	static const int ms_JumpAfterXLines;
	static const int ms_PosTable;

	std::vector<CColumn> m_ColumnEnum;
	CColumnHead m_ColumnHead;

	template<class T>
	CTable(const T& value,const int& nb_row) :
	CValue(value),CDataToShow(ms_PosTable),m_ColumnEnum(nb_row) { }
		
	virtual void write(std::ostream& stream) const;

	template<class T> void ShowBases(const T& bases_enum) {
		InsertNewValue(bases_enum.size());
		m_ColumnHead.m_BoxEnum.push_back(CBox("LABEL",CBox::ms_ColumnSize_Label));
		m_ColumnHead.m_BoxEnum.push_back(CBox("G_L",CBox::ms_ColumnSize_List));
		m_ColumnHead.m_BoxEnum.push_back(CBox("L_L",CBox::ms_ColumnSize_List));

		for(int i=0;i<bases_enum.size();i++) {
			m_ColumnEnum[i].m_BoxEnum.push_back(CBox(bases_enum[i].label,CBox::ms_ColumnSize_Label));

			CBox global_labels("",CBox::ms_ColumnSize_List);
			for(int j=0;j<bases_enum[i].vertices_global.size();j++) {			
				global_labels.InsertNewValue(bases_enum[i].vertices_global[j]);
			}
			m_ColumnEnum[i].m_BoxEnum.push_back(global_labels);
		
			CBox local_labels("",CBox::ms_ColumnSize_List);
			for(j=0;j<bases_enum[i].vertices_local.size();j++) {			
				local_labels.InsertNewValue(bases_enum[i].vertices_local[j]);
			}
			m_ColumnEnum[i].m_BoxEnum.push_back(local_labels);
		}
	}
};

#endif