/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 18/04/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.out.intern.h"



hyp_NAMESPACE_BEGIN_HYPOUT
hyp_NAMESPACE_BEGIN(stdfile)


CStdImplFile::CStdImplFile(hyp_ker::IUnknown* pUnknownOuter) :
CUnknown(pUnknownOuter)
{
	m_pStream=0;
}

CStdImplFile::~CStdImplFile()
{
	hyp_TRACE(("~CStdImpl"));
}


hyp_ker::com_result CStdImplFile::NondelegatingQueryInterface(const hyp_ker::hypIID& iid,void** ppv)
{
	if(iid==IID_hypIShowMeshObject) {
		return FinishQI(static_cast<IShowMeshObject*>(this),ppv);
	} else {
		return CUnknown::NondelegatingQueryInterface(iid,ppv);
	}
}
//DOXYTAG

hyp_ker::com_result CStdImplFile::CreateInstance(hyp_ker::IUnknown* pUnknownOuter,hyp_ker::CUnknown** ppNewComponent)
{
	*ppNewComponent=new CStdImplFile(pUnknownOuter);
	return hyp_ker::com_ok;
}


hyp_ker::com_result CStdImplFile::Init(void)
{
	return hyp_ker::com_ok;
}

void CStdImplFile::FinalRelease(void)
{
	CloseFile();
	CUnknown::FinalRelease();
}

//////
//file management
void CStdImplFile::OpenFile(const std::string& file)
{
	CloseFile();

	//creation du fichier
	//FIXTODO : systeme de path multi-os
	std::string path;
	std::string __file;
	int pos=file.find('\\');
	path.insert(0,file.c_str(),pos+1);
	__file.insert(0,file,pos+1,file.size()-(pos+1));
	hyp_ker::CDir dir;
	dir.Open(path);

	//MODIF
	dir.CreateFile(__file);
	//hyp_THROW_COM2(hyp_ker::com_ios,("%s couldn't be created",file.c_str()) );
	
	//creation du stream
	m_pStream=new std::fstream(file.c_str());
	*m_pStream<<std::fixed<<std::setprecision(10);
	m_pStream->setf(std::ios_base::floatfield);
	*m_pStream<<std::boolalpha;

	*m_pStream<<GetHead();
}

void CStdImplFile::CloseFile(void)
{
	if(m_pStream) *m_pStream<<"\n\nEOF";
	delete m_pStream;
	m_pStream=0;
}

void CStdImplFile::SetOutput(const std::string& FileName)
{
	OpenFile(FileName);
}

void CStdImplFile::SetOutput(const std::ostream& Stream)
{
	CloseFile();
	m_pStream=new std::ostream(Stream);
}

void CStdImplFile::CloseOutput()
{
	CloseFile();
}


///////
void CStdImplFile::ShowAll()
{
	//FIXTODO : verif si on peut afficher masks, displac et forces
	ShowNodes();
	ShowSides();
	ShowElements();
	ShowMatrixDim();
	ShowGlobalForces();
	ShowGlobalDisplacements();	
}

void CStdImplFile::ShowNodes()
{
	TEST_OUTPUT();

	CMeshData data(m_spUnknownMesh);	
	CMeshData::SVerticesEnum vertices_enum=data.GetVerticesEnum();

	//creation de la table
	CTable NodeTable("NODE LIST",vertices_enum.size());
	NodeTable.InsertNewValue(vertices_enum.size());
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("LABEL",CBox::ms_ColumnSize_Label));
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("X",CBox::ms_ColumnSize_Real));
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("Y",CBox::ms_ColumnSize_Real));
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("Z",CBox::ms_ColumnSize_Real));
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("VISIBLE",CBox::ms_ColumnSize_Bool));
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("DOF_X",CBox::ms_ColumnSize_Bool));
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("DOF_Y",CBox::ms_ColumnSize_Bool));
	NodeTable.m_ColumnHead.m_BoxEnum.push_back(CBox("DOF_Z",CBox::ms_ColumnSize_Bool));


	for(int i=0;i<vertices_enum.size();i++) {
		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].label,CBox::ms_ColumnSize_Label));

		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].coord[0],CBox::ms_ColumnSize_Real));
		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].coord[1],CBox::ms_ColumnSize_Real));
		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].coord[2],CBox::ms_ColumnSize_Real));

		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].b_visible,CBox::ms_ColumnSize_Bool));

		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].dof[0],CBox::ms_ColumnSize_Bool));
		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].dof[1],CBox::ms_ColumnSize_Bool));
		NodeTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(vertices_enum[i].dof[2],CBox::ms_ColumnSize_Bool));
	}

	NodeTable.write(*m_pStream);
}


void CStdImplFile::ShowSides()
{
	TEST_OUTPUT();
	
	CMeshData data(m_spUnknownMesh);	
	CMeshData::SSidesEnum sides_enum=data.GetSidesEnum();

	CTable SideTable("SIDE LIST",sides_enum.size());
	SideTable.ShowBases(sides_enum);	
	SideTable.write(*m_pStream);
}

void CStdImplFile::ShowElements()
{
	TEST_OUTPUT();

	CMeshData data(m_spUnknownMesh);	
	CMeshData::SElementsEnum elements_enum=data.GetElementsEnum();

	CTable ElementTable("ELEMENT LIST",elements_enum.size());	
	ElementTable.ShowBases(elements_enum);
	
	ElementTable.m_ColumnHead.m_BoxEnum.push_back(CBox("YOUNG",CBox::ms_ColumnSize_Real));
	ElementTable.m_ColumnHead.m_BoxEnum.push_back(CBox("POISSON",CBox::ms_ColumnSize_Real));
	
	for(int i=0;i<elements_enum.size();i++) {
		ElementTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(elements_enum[i].young,CBox::ms_ColumnSize_Real));
		ElementTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(elements_enum[i].poisson,CBox::ms_ColumnSize_Real));			
	}
	ElementTable.write(*m_pStream);
}

void CStdImplFile::ShowMatrixDim()
{
	TEST_OUTPUT();

	const hyp_fem::IMatrix* p_matrix=0;
	p_matrix=GetFEOMesh()->K();

	CTable table("MATRIX DIM",0);
	table.write(*m_pStream);
	*m_pStream<<"Rows :"<<p_matrix->GetNbRow()<<" , "
		<<"Columns :"<<p_matrix->GetNbColumn()<<std::endl;
}

void CStdImplFile::ShowMatrix()
{
	TEST_OUTPUT();
	
	CTable table("MATRIX (Maple format)",0);
	table.write(*m_pStream);

	const hyp_fem::IMatrix* p_matrix=0;
	p_matrix=GetFEOMesh()->K();
	
	int i,j;
	t_size row,col;
	row=p_matrix->GetNbRow();
	col=p_matrix->GetNbColumn();

	*m_pStream<<"K:=array(1.."<<row<<",1.."<<col<<", [\n";
	for(i=0;i<row;i++) {
		*m_pStream<<"[";
		for(j=0;j<col;j++) {
			*m_pStream<<p_matrix->GetValue(i,j);
			if(j!=col-1) *m_pStream<<",";
		}
		if(i!=row-1) {
			*m_pStream<<"],\n";
		} else {
			*m_pStream<<"]\n";
		}
	}
	
	*m_pStream<<" ]):;\n";
}

void CStdImplFile::ShowMask(const t_mask_array& mask_array,const std::string& commentary)
{
	TEST_OUTPUT();	

	CTable MaskTable(commentary,mask_array.size());
	MaskTable.InsertNewValue(mask_array.size());
	MaskTable.m_ColumnHead.m_BoxEnum.push_back(CBox("CMPT",CBox::ms_ColumnSize_Label));
	MaskTable.m_ColumnHead.m_BoxEnum.push_back(CBox("M",CBox::ms_ColumnSize_Bool));
			
	for(int i=0;i<mask_array.size();i++) {
		MaskTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(i,CBox::ms_ColumnSize_Label));
		MaskTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(mask_array[i],CBox::ms_ColumnSize_Bool));
	}

	MaskTable.write(*m_pStream);
}

void CStdImplFile::ShowArray(const t_real_array& real_array,const std::string& commentary)
{
	TEST_OUTPUT();
	
	CTable ArrayTable(commentary,(real_array.size()/3));
	ArrayTable.InsertNewValue(real_array.size());
	ArrayTable.m_ColumnHead.m_BoxEnum.push_back(CBox("CMPT",CBox::ms_ColumnSize_Label));
	ArrayTable.m_ColumnHead.m_BoxEnum.push_back(CBox("X",CBox::ms_ColumnSize_Real));
	ArrayTable.m_ColumnHead.m_BoxEnum.push_back(CBox("Y",CBox::ms_ColumnSize_Real));
	ArrayTable.m_ColumnHead.m_BoxEnum.push_back(CBox("Z",CBox::ms_ColumnSize_Real));
		
	for(int i=0;i<real_array.size()/3;i++) {
		ArrayTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(i,CBox::ms_ColumnSize_Label));
		for(int j=0;j<3;j++) {
			ArrayTable.m_ColumnEnum[i].m_BoxEnum.push_back(CBox(real_array[3*i+j],CBox::ms_ColumnSize_Real));
		}
	}
	ArrayTable.write(*m_pStream);
}

void CStdImplFile::ShowForceMask()
{
	ShowMask(GetFEOMesh()->RealTimeMask(),"FORCE MASK");
}

void CStdImplFile::ShowRenderMask()
{
	ShowMask(GetFEOMesh()->RenderMask(),"RENDER MASK");
}

void CStdImplFile::ShowForces()
{
	ShowArray(GetFEOMesh()->RealTimeForce(),"COMPRESSED FORCE VECTOR");
}

void CStdImplFile::ShowGlobalForces()
{
	ShowArray(GetFEOMesh()->GetGlobalRealTimeForce(),"GLOBAL FORCE VECTOR");
}

void CStdImplFile::ShowDisplacements()
{
	ShowArray(GetFEOMesh()->Displacement(),"COMPRESSED DISPL. VECTOR");
}

void CStdImplFile::ShowGlobalDisplacements()
{
	ShowArray(GetFEOMesh()->GetGlobalDisplacement(),"GLOBAL DISPL. VECTOR");
}
hyp_NAMESPACE_END(stdfile)
hyp_NAMESPACE_END_HYPOUT