/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
 
 Creation 27/03/01 Pierre Rebours
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.out.intern.h"

hyp_NAMESPACE_BEGIN_HYPOUT


CMeshData::CMeshData(hyp_ker::IUnknown* p_geometric_object)
{
	m_spMeshMesh=m_spBaseMesh=m_spObjectMesh=p_geometric_object;
}

//retourne un tableau avec la liste des labels
t_label_enum CMeshData::GetLabelsForContainer(hyp_ker::IUnknown* p_cont,hyp_ker::IUnknown* p_cont_absolu)
{
	t_label_enum tab;
	hyp_fem::t_spContainerKey spContKey_Absolu;
	if(p_cont_absolu) {
		spContKey_Absolu=p_cont_absolu;
	} else {
		spContKey_Absolu=p_cont;
	}
	
	hyp_fem::t_spContainerIterator spContItor=p_cont;
	hyp_ker::IUnknown* p_content=0;
	try {
		p_content=spContItor->SetBeginSelection();
		while(1) {
			tab.push_back(spContKey_Absolu->GetKey(p_content));
			p_content=spContItor->IncrSelection();
		}
	} hyp_CATCH_ITOR

	return tab;
}

CMeshData::SVerticesEnum CMeshData::GetVerticesEnum(void)
{
	t_size nVertices=m_spBaseMesh->GetNumberOfVertices();
	
	SVerticesEnum vertices_enum;
	vertices_enum.reserve(nVertices);
	
	hyp_fem::t_spContainerIterator spContItor=m_spBaseMesh->GetContainerOfVertices();
	hyp_fem::t_spGeometricVertex p_vertex;
	int i=0;
	try{
		p_vertex=spContItor->SetBeginSelection();
		while(1) {
			SVertex vertex;
			vertex.label=GetLabelVertex(p_vertex);
			for(int j=0;j<3;j++) {
				vertex.coord[j]=p_vertex->GetCoord(j);
			}
			
			vertex.b_visible=p_vertex->IsVisible();

			hyp_fem::t_spFEOBoundaryConditions spNode=p_vertex;
			for(j=0;j<3;j++) {
				vertex.dof[j]=spNode->IsFixed(j);
			}			

			vertices_enum.push_back(vertex);
			p_vertex=spContItor->IncrSelection();
		}
	} hyp_CATCH_ITOR

	return vertices_enum;
}

CMeshData::SSidesEnum CMeshData::GetSidesEnum(void)
{
	t_size nSides=m_spObjectMesh->GetNumberOfSides();
	
	SSidesEnum sides_enum;
	sides_enum.reserve(nSides);

	hyp_fem::t_spContainerIterator spContItor=m_spObjectMesh->GetContainerOfSides();
	hyp_ker::IUnknown* p_side=0;

	try{
		p_side=spContItor->SetBeginSelection();
		while(1) {
			SSide side;
			side.label=GetLabelSide(p_side);
			side.vertices_global=GetLabelsForContainer(p_side,m_spBaseMesh->GetContainerOfVertices());
			side.vertices_local=GetLabelsForContainer(p_side);
			
			sides_enum.push_back(side);

			p_side=spContItor->IncrSelection();
		}
	} hyp_CATCH_ITOR

	return sides_enum;
}

CMeshData::SElementsEnum CMeshData::GetElementsEnum(void)
{
	t_size nElements=m_spMeshMesh->GetNumberOfElements();

	SElementsEnum elements_enum;
	elements_enum.reserve(nElements);

	hyp_fem::t_spContainerIterator spContItor=m_spMeshMesh->GetContainerOfElements();
	hyp_ker::IUnknown* p_element=0;

	try{
		p_element=spContItor->SetBeginSelection();
		while(1) {
			SElement element;
			element.label=GetLabelElement(p_element);
			element.vertices_global=GetLabelsForContainer(p_element,m_spBaseMesh->GetContainerOfVertices());
			element.vertices_local=GetLabelsForContainer(p_element);
			
			hyp_fem::t_spFEOElement spElement=p_element;
			element.young=spElement->GetMaterial()->GetYoung();
			element.poisson=spElement->GetMaterial()->GetPoisson();

			elements_enum.push_back(element);

			p_element=spContItor->IncrSelection();
		}
	} hyp_CATCH_ITOR

	return elements_enum;
}


t_label CMeshData::GetLabelVertex(hyp_ker::IUnknown* p_vertex)
{
	hyp_fem::t_spContainerKey spContKey=m_spBaseMesh->GetContainerOfVertices();
	return spContKey->GetKey(p_vertex);
}

t_label CMeshData::GetLabelSide(hyp_ker::IUnknown* p_side)
{
	hyp_fem::t_spContainerKey spContKey=m_spObjectMesh->GetContainerOfSides();
	return spContKey->GetKey(p_side);
}

t_label CMeshData::GetLabelElement(hyp_ker::IUnknown* p_element)
{
	hyp_fem::t_spContainerKey spContKey=m_spMeshMesh->GetContainerOfElements();
	return spContKey->GetKey(p_element);
}


hyp_NAMESPACE_END_HYPOUT


