/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypVisual.
 HypVisual is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.visual.intern.h"

hyp_NAMESPACE_BEGIN_HYPVISUAL



CViscosityDialog::CViscosityDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CViscosityDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CViscosityDialog)
	//}}AFX_DATA_INIT
}


void CViscosityDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CViscosityDialog)
	
	DDX_Text(pDX, IDC_EXTRA_DAMPING, m_Damping);
	DDV_MinMaxDouble(pDX,m_Damping,0.0f,10000.0f);

	DDX_Text(pDX, IDC_EXTRA_OSCIL, m_NumberOfOscillation);
	DDV_MinMaxInt(pDX,m_NumberOfOscillation,1,10000);

	DDX_Text(pDX, IDC_NSTEPS, m_NumberOfSteps);
	DDV_MinMaxInt(pDX,m_NumberOfSteps,1,10000);

	DDX_Text(pDX, IDC_DELAY, m_Delay);
	DDV_MinMaxInt(pDX,m_Delay,1,10000);	

	DDX_Text(pDX, IDC_MULTICOEF, m_MultiCoef);

	DDX_Check(pDX, IDC_REPEAT, m_bRepeat);	

	//}}AFX_DATA_MAP
}

BOOL CViscosityDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	if(m_Function==CSimulationTool::SinusFunction) {
		CheckDlgButton(IDC_SINUSMODE,1);
	} else if(m_Function==CSimulationTool::SinusExpFunction) {
		CheckDlgButton(IDC_RADIO3,1);
	} else if(m_Function==CSimulationTool::LinearFunction) {
		CheckDlgButton(IDC_LINEARMODE,1);
	}

	ShowExtraExpDamping(m_Function==CSimulationTool::SinusExpFunction);	
	
	return TRUE;
}

void CViscosityDialog::OnSetLinearMode()
{
	m_Function=CSimulationTool::LinearFunction;
	ShowExtraExpDamping(false);
}

void CViscosityDialog::OnSetSinusMode()
{
	m_Function=CSimulationTool::SinusFunction;
	ShowExtraExpDamping(false);
}

void CViscosityDialog::OnSetExpDamping()
{
	m_Function=CSimulationTool::SinusExpFunction;
	ShowExtraExpDamping(true);
}

void CViscosityDialog::ShowExtraExpDamping(bool bShow)
{
	int param=bShow ? SW_SHOW : SW_HIDE;
	
	GetDlgItem(IDC_EXTRA_TITLE)->ShowWindow(param);
	GetDlgItem(IDC_EXTRA_DAMPING_TEXT)->ShowWindow(param);
	GetDlgItem(IDC_EXTRA_DAMPING)->ShowWindow(param);
	GetDlgItem(IDC_EXTRA_OSCIL_TEXT)->ShowWindow(param);
	GetDlgItem(IDC_EXTRA_OSCIL)->ShowWindow(param);
}


BEGIN_MESSAGE_MAP(CViscosityDialog, CDialog)
	//{{AFX_MSG_MAP(CViscosityDialog)
		//{{AFX_MSG_MAP(COptionsDialog)
	ON_BN_CLICKED(IDC_SINUSMODE, OnSetSinusMode)
	ON_BN_CLICKED(IDC_LINEARMODE, OnSetLinearMode)
	ON_BN_CLICKED(IDC_EXPDAMPINGMODE, OnSetExpDamping)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CViscosityDialog message handlers


BOOL CViscosityDialog::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	return CDialog::Create(IDD, pParentWnd);	
}

hyp_NAMESPACE_END_HYPVISUAL




