/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypVisual.
 HypVisual is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include "hyp.visual.intern.h"



hyp_NAMESPACE_BEGIN_HYPVISUAL

//CCheckTool
class CCheckTool
{
CRect m_Rect;
t_real m_PlanCoord;
t_size m_Plan;
t_size m_WidthAxe;
t_size m_HeightAxe;
public:
	CCheckTool(const CRect&,const t_real&,const t_size&);
	bool IsInRect(hyp_fem::IGeometricVertex*);
	bool IsOnRectBoundary(hyp_fem::IGeometricVertex*);
	bool IsInRectCorner(hyp_fem::IGeometricVertex*);
private:
	bool IsOnRectBetweenHorizontalEdge(hyp_fem::IGeometricVertex*);
	bool IsOnRectBetweenVerticalEdge(hyp_fem::IGeometricVertex*);
	bool IsOnRectHorizontalEdge(hyp_fem::IGeometricVertex*);
	bool IsOnRectVerticalEdge(hyp_fem::IGeometricVertex*);
};

CCheckTool::CCheckTool(const CRect& Rect,const t_real& PlanCoord,const t_size& Plan) :
m_Rect(Rect),
m_PlanCoord(PlanCoord),
m_Plan(Plan)
{
	switch(m_Plan) {
	case 0:
		m_WidthAxe=1;
		m_HeightAxe=2;
		break;
	case 1:
		m_WidthAxe=0;
		m_HeightAxe=2;
		break;
	case 2:
		m_WidthAxe=0;
		m_HeightAxe=1;
		break;
	default:
		hyp_ASSERT(0);
	}
}

bool CCheckTool::IsOnRectBetweenHorizontalEdge(hyp_fem::IGeometricVertex* pVertex)
{
	return (pVertex->GetCoord(m_Plan)==m_PlanCoord) &&
		(pVertex->GetCoord(m_WidthAxe)>=m_Rect.left && pVertex->GetCoord(m_WidthAxe)<=m_Rect.right);
}

bool CCheckTool::IsOnRectBetweenVerticalEdge(hyp_fem::IGeometricVertex* pVertex)
{
	return (pVertex->GetCoord(m_Plan)==m_PlanCoord) &&
		(pVertex->GetCoord(m_HeightAxe)>=m_Rect.bottom && pVertex->GetCoord(m_HeightAxe)<=m_Rect.top);
}

bool CCheckTool::IsOnRectHorizontalEdge(hyp_fem::IGeometricVertex* pVertex)
{
	return IsOnRectBetweenHorizontalEdge(pVertex) &&
		(pVertex->GetCoord(m_HeightAxe)==m_Rect.bottom || pVertex->GetCoord(m_HeightAxe)==m_Rect.top);
}

bool CCheckTool::IsOnRectVerticalEdge(hyp_fem::IGeometricVertex* pVertex)
{
	return IsOnRectBetweenVerticalEdge(pVertex) &&
		(pVertex->GetCoord(m_WidthAxe)==m_Rect.left || pVertex->GetCoord(m_WidthAxe)==m_Rect.right);
}

bool CCheckTool::IsInRect(hyp_fem::IGeometricVertex* pVertex)
{
	return IsOnRectBetweenHorizontalEdge(pVertex) && IsOnRectBetweenVerticalEdge(pVertex);
}

bool CCheckTool::IsOnRectBoundary(hyp_fem::IGeometricVertex* pVertex)
{
	return IsOnRectHorizontalEdge(pVertex) || IsOnRectVerticalEdge(pVertex);
}

bool CCheckTool::IsInRectCorner(hyp_fem::IGeometricVertex* pVertex)
{
	return IsOnRectHorizontalEdge(pVertex) && IsOnRectVerticalEdge(pVertex);	
	/*
	return (pVertex->GetCoord(m_WidthAxe)==m_Rect.left || pVertex->GetCoord(m_WidthAxe)==m_Rect.right) &&
		(pVertex->GetCoord(m_HeightAxe)==m_Rect.bottom || pVertex->GetCoord(m_HeightAxe)==m_Rect.top);
	*/
}

IMPLEMENT_DYNCREATE(CHypVisualDoc, CDocument)

BEGIN_MESSAGE_MAP(CHypVisualDoc, CDocument)
	//{{AFX_MSG_MAP(CHypVisualDoc)
	ON_COMMAND(ID_VIEWDETAILS_SHOWMATRIX, OnShowMatrix)
	ON_COMMAND(ID_SIMULATION_FORCES, OnSetForces)
	ON_COMMAND(ID_SHOWDETAILS_CLEARFORCES, OnClearForces)
	ON_COMMAND(ID_SHOWDETAILS_SHOWNODES, OnShowNodes)
	ON_COMMAND(ID_SHOWDETAILS_SHOWSIDES, OnShowSides)
	ON_COMMAND(ID_SHOWDETAILS_SHOWELEMENTS, OnShowElements)
	ON_COMMAND(ID_SHOWDETAILS_SHOWRENDERMASK, OnShowRenderMask)
	ON_COMMAND(ID_SHOWDETAILS_SHOWFORCEMASK, OnShowForceMask)
	ON_COMMAND(ID_SHOWDETAILS_SHOWDISPLACEMENTS, OnShowDisplacements)
	ON_COMMAND(ID_SHOWDETAILS_SHOWGLOBALDISPLACEMENTS, OnShowGlobalDisplacements)
	ON_COMMAND(ID_SHOWDETAILS_SHOWFORCES, OnShowForces)
	ON_COMMAND(ID_SHOWDETAILS_SHOWGLOBALFORCES, OnShowGlobalForces)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


CHypVisualDoc::CHypVisualDoc()
{
	CreateMesh();
//	m_bUpdate=false;

}

CHypVisualDoc::~CHypVisualDoc()
{
}



BOOL CHypVisualDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	CWaitCursor wait;
	try	{
		hyp_TRACE( ("Load Mesh from %s",lpszPathName) );
		hyp_out::t_spD3DMeshObject spD3DMesh;
		spD3DMesh.CreateInstance(hyp_out::CLSID_hypCObjectXFile,0);

		spD3DMesh->SetObject(m_spUnknownMesh);
		spD3DMesh->LoadD3DHypMesh(lpszPathName);

		m_spMeshMesh->InitMasks();
	} catch(hyp_ker::CComException hr) {
		hyp_NOTIFY( ("Error during the loading of the file") );
		return FALSE;
	}	
	
	return TRUE;
}

BOOL CHypVisualDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	CWaitCursor wait;
	try	{
		hyp_TRACE( ("Save Geometric Mesh from %s",lpszPathName) );

		m_spMeshMesh->PostRender();

		hyp_out::t_spD3DMeshObject spD3DMesh;
		spD3DMesh.CreateInstance(hyp_out::CLSID_hypCObjectXFile,0);

		spD3DMesh->SetObject(m_spUnknownMesh);
		spD3DMesh->SaveD3DMeshBuilder(lpszPathName);
		
	} catch(hyp_ker::CComException hr) {
		hyp_NOTIFY( ("Error during the saving of the file") );
		return FALSE;
	}	
	
	return TRUE;
}


BOOL CHypVisualDoc::OnNewDocument() 
{
	const int buff_size=1000;
	char buff[buff_size];
	if(::GetModuleFileName(NULL,buff,buff_size)) {
		std::string str(buff);
		int pos=str.find_last_of('\\');
		std::string path;
		path.insert(0,str.c_str(),pos);
		path+="\\default_file.hyp";
		SetPathName(path.c_str());
		SetTitle("default_file.hyp");
		return OnOpenDocument(path.c_str());
	} else {
		hyp_NOTIFY( ("GetModuleFileName has returned an error") );
		return FALSE;
	}
}


void CHypVisualDoc::CreateMesh(void)
{
	m_spUnknownMesh.CreateInstance(hyp_fem::CLSID_hypCFEOMesh,0);
	InitMesh(m_spUnknownMesh);
}

void CHypVisualDoc::InitMesh(hyp_ker::IUnknown* p_mesh)
{
	m_spUnknownMesh=p_mesh;
	m_spMeshMesh=m_spObjectMesh=m_spBaseMesh=m_spUnknownMesh;
}

////////
//show functions...
void CHypVisualDoc::OnShowMatrix() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_matrix,m_spUnknownMesh);
	aDlg.DoModal();	
}
void CHypVisualDoc::OnShowNodes() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_nodes,m_spUnknownMesh);
	aDlg.DoModal();	
}
void CHypVisualDoc::OnShowSides() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_sides,m_spUnknownMesh);
	aDlg.DoModal();
}
void CHypVisualDoc::OnShowElements() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_elements,m_spUnknownMesh);
	aDlg.DoModal();	
}
void CHypVisualDoc::OnShowRenderMask() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_rendermask,m_spUnknownMesh);
	aDlg.DoModal();	
}
void CHypVisualDoc::OnShowForceMask() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_forcemask,m_spUnknownMesh);
	aDlg.DoModal();
}
void CHypVisualDoc::OnShowDisplacements() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_local_displacements,m_spUnknownMesh);
	aDlg.DoModal();
}
void CHypVisualDoc::OnShowGlobalDisplacements()
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_global_displacements,m_spUnknownMesh);
	aDlg.DoModal();
}
void CHypVisualDoc::OnShowForces() 
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_local_forces,m_spUnknownMesh);
	aDlg.DoModal();	
}
void CHypVisualDoc::OnShowGlobalForces()
{
	CShowDialog aDlg;
	aDlg.InitFileBuffer(CShowDialog::show_global_forces,m_spUnknownMesh);
	aDlg.DoModal();
}


void CHypVisualDoc::OnSetForces() 
{
	CForcesDialog aDlg;
	if(aDlg.DoModal()==IDOK) {
		switch(aDlg.m_TypeOfStress) {
			case CForcesDialog::ponctual:
			m_spMeshMesh->SetForce(aDlg.m_RegionX,aDlg.m_RegionY,aDlg.m_RegionZ,
				aDlg.m_InX==TRUE ? true : false,
				aDlg.m_InY==TRUE ? true : false,
				aDlg.m_InZ==TRUE ? true : false,
				aDlg.m_ForceX,aDlg.m_ForceY,aDlg.m_ForceZ);
			break;
			case CForcesDialog::pression: {
				if (aDlg.m_InX+aDlg.m_InY+aDlg.m_InZ>1) hyp_ASSERT( "A pression is applied only on a surface" );
				if (aDlg.m_InX+aDlg.m_InY+aDlg.m_InZ==0) {
					hyp_NOTIFY( ("No surface has been selected" ));
					break;
				}
				
				t_size plan=0;
				t_real plan_coord=0;
				if(aDlg.m_InX) {
					plan=0;
					plan_coord=aDlg.m_RegionX;
				} else if(aDlg.m_InY) {
					plan=1;
					plan_coord=aDlg.m_RegionY;
				} else if(aDlg.m_InZ) {
					plan=2;
					plan_coord=aDlg.m_RegionZ;
				} else {
					hyp_ASSERT(0);
				}
				
				//CRect(l,t,r,b)
				CCheckTool Check(CRect(aDlg.m_O_1,
					aDlg.m_l+aDlg.m_O_2,
					aDlg.m_L+aDlg.m_O_1,
					aDlg.m_O_2),plan_coord,plan);

				hyp_fem::t_spContainerKey spContKeyVertices=m_spBaseMesh->GetContainerOfVertices();
				hyp_fem::t_spContainerIterator spContItorVertices=spContKeyVertices;

				try {
					int j=0;
					hyp_fem::t_spGeometricVertex spVertex=spContItorVertices->SetBeginSelection();
					while(1) {
						t_real coord[3];
						for(int i=0;i<3;i++) {
							coord[i]=spVertex->GetCoord(i);
						}

						t_real Fx=aDlg.m_ForceX;
						t_real Fy=aDlg.m_ForceY;
						t_real Fz=aDlg.m_ForceZ;
						if(Check.IsInRectCorner(spVertex)) {
							Fx/=4;Fy/=4;Fz/=4;
						} else if(Check.IsOnRectBoundary(spVertex)) {
							Fx/=2;Fy/=2;Fz/=2;
						} else if(Check.IsInRect(spVertex)) {
							//on fait rien...
						} else {
							Fx=0;Fy=0;Fz=0;
						}
						m_spMeshMesh->SetForce(spContKeyVertices->GetKey(spVertex),Fx,Fy,Fz);

						spVertex=spContItorVertices->IncrSelection();
						j++;
					}
				} hyp_CATCH_ITOR
				break;
			}
			default:
				hyp_ASSERT(0);
		}
	}	
}

void CHypVisualDoc::OnClearForces()
{
	m_spMeshMesh->ClearRealTimeForce();
}


void CHypVisualDoc::GetXFileName(std::string& str)
{
	//on cherche l'extension
	int pos=str.find_last_of('.');
	if(pos>=0) {
		str.replace(pos,str.size()-pos,".x");
	} else {
		str+=".x";
	}
}


void CHypVisualDoc::OnFileSaveAs() 
{
	std::string str(GetPathName());
	GetXFileName(str);
	OnSaveDocument(str.c_str());	
}

void CHypVisualDoc::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	std::string str("Save as ");
	str+=GetTitle();
	GetXFileName(str);
	pCmdUI->SetText(str.c_str());	
}


hyp_NAMESPACE_END_HYPVISUAL
