/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypVisual.
 HypVisual is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.visual.intern.h"

#include<math.h>

hyp_NAMESPACE_BEGIN_HYPVISUAL


std::string GetLocalPath(const std::string& FileName)
{
	const int buff_size=1000;
	char buff[buff_size];	
	std::string path;

	if(::GetModuleFileName(NULL,buff,buff_size)) {
		std::string str(buff);
		int pos=str.find_last_of('\\');
		path.insert(0,buff,pos);
		path+='\\';
		path+=FileName;		
	}
	return path;
}


const t_real PI=3.14;
int CSimulationTool::ms_NumberOfOscillation =10;
double CSimulationTool::ms_Damping=1;

double CSimulationTool::SinusFunction(const double& t,const double& Max)
{
	hyp_ASSERT(t<=Max && t>=0);
	
	return ::sin((2*PI*t)/Max);
}

double CSimulationTool::LinearFunction(const double& t,const double& Max)
{
	hyp_ASSERT(t<=Max && t>=0);
	
	return t/Max;
}

double CSimulationTool::SinusExpFunction(const double& t,const double& Max)
{
	hyp_ASSERT(t<=Max && t>=0);

	return ::sin((2*PI*t)/(Max/ms_NumberOfOscillation))*::exp((-t*ms_Damping)/Max);
}

void CALLBACK CSimulationTool::TimeProc(UINT IDTimer,UINT,DWORD dwUser,DWORD,DWORD)
{
	hyp_ASSERT(dwUser);
	CSimulationTool* pSimulation=(CSimulationTool*)(dwUser);

	
	if(pSimulation->GetState()==CSimulationTool::play) {

	t_real incr=pSimulation->m_InterpolFunction(ms_StepsDone,pSimulation->m_NumberOfSteps);
	hyp_ASSERT(incr<=1&&incr>=-1);

	//proportion
	incr*=pSimulation->m_MultiCoef;

	for(int i=0;i<pSimulation->m_Displacement.size();i+=3) {
		D3DVECTOR coord;
		pSimulation->m_pMeshCopy->GetVertex(i/3,&coord);
		pSimulation->m_pMesh->SetVertex(i/3,
			coord.x+pSimulation->m_Displacement[i]*incr,
			coord.y+pSimulation->m_Displacement[i+1]*incr,
			coord.z+pSimulation->m_Displacement[i+2]*incr);
	}
	
	if(ms_StepsDone>=pSimulation->m_NumberOfSteps) {
		if(pSimulation->m_bRepeat) {
			ms_StepsDone=0;
		} else {
			pSimulation->m_State=stop;	//astuce pour finir proprement l'interpolation
			::PostMessage(CTool::GetHViewWindow(),ID_STOPSIMULATION,0,0);
		}
	}

	CTool::InvalidateView();
	ms_StepsDone++;

	} else {
	 	hyp_ASSERT(pSimulation->GetState()==CSimulationTool::stop);
	}	
}


hyp_NAMESPACE_END_HYPVISUAL