/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypVisual.
 HypVisual is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.visual.intern.h"

hyp_NAMESPACE_BEGIN_HYPVISUAL

CHypVisualView* CTool::m_pView=0;
t_real CSimulationTool::ms_StepsDone=0;


CHypVisualDoc* CTool::GetViewDocument(void)
{
	return m_pView->GetDocument();
}

HWND CTool::GetHViewWindow(void)
{
	return m_pView->m_hWnd;
}

void CTool::InvalidateView(void)
{
	m_pView->Invalidate(FALSE);
}


void CTool::UpdateStatusBar(void)
{
	m_pView->OnUpdateStatusBar();
	
	//m_pView->PostMessage(ID_UPDATE_STATUSBAR);
}


//CRotationTool
	
CRotationTool::CRotationTool()
{
	m_Key=VK_SPACE;
	m_bRotate=false;	
}
	
void CRotationTool::RotateBegin(UINT Key)
{
	if(Key==m_Key) m_bRotate=true;
	UpdateStatusBar();
}

void CRotationTool::RotateEnd(UINT Key)
{
	if(Key==m_Key) m_bRotate=false;
	UpdateStatusBar();
}


//CSimulationTool
CSimulationTool::CSimulationTool() :
m_Displacement(0),
m_bRepeat(false),
m_InterpolFunction(LinearFunction),
m_MultiCoef(1),
m_NumberOfSteps(10),
m_pMesh(0),
m_State(stop),
m_Delay(100),
m_TimeID(0)
{
	
}

CSimulationTool::~CSimulationTool()
{
	Stop();
}

void CSimulationTool::CreateCopyMesh(CRenderTool& RenderTool)
{
	hyp_CHECK_HR( RenderTool.GetD3DRM()->CreateMeshBuilder(&m_pMeshCopy) );
}

void CSimulationTool::Stop(void)
{
	KillTimer();
	
	m_State=stop;
	UpdateStatusBar();
}

void CSimulationTool::Play(void)
{
	Stop();
	hyp_ASSERT(m_TimeID==0);
	m_TimeID=::timeSetEvent(m_Delay,10,TimeProc,(DWORD)(this),TIME_PERIODIC);
	if(!m_TimeID) hyp_FATAL( ("Failure during the creation of the timer") );

	m_State=play;
	UpdateStatusBar();
}

void CSimulationTool::KillTimer(void)
{
	if(m_TimeID) {
		::timeKillEvent(m_TimeID);
		m_TimeID=0;
	}
}

void CSimulationTool::Pause(void)
{
	KillTimer();
	
	m_State=pause;
	UpdateStatusBar();
}

void CSimulationTool::Start(void)
{
	ms_StepsDone=0;
	Play();
}

void CSimulationTool::SetInterpolationParameters(bool Repeat,PTR_FUNCTION Function,double MultiCoef,int NumberOfSteps, int Delay)
{
	m_bRepeat=Repeat;
	m_InterpolFunction=Function;
	m_MultiCoef=MultiCoef;
	m_NumberOfSteps=NumberOfSteps;
	m_Delay=Delay;
}

void CSimulationTool::GetInterpolationParameters(BOOL& Repeat,PTR_FUNCTION& Function,double& MultiCoef,int& NumberOfSteps,int& Delay) const
{
	Repeat= (m_bRepeat ? TRUE : FALSE) ;
	Function=m_InterpolFunction;
	MultiCoef=m_MultiCoef;
	NumberOfSteps=m_NumberOfSteps;
	Delay=m_Delay;
}

void CSimulationTool::SetInterpolationData(IDirect3DRMMeshBuilder3* pMesh,const t_real_array& Displacement)
{
	m_pMesh=pMesh;
	m_Displacement=Displacement;

	hyp_CHECK_HR( m_pMeshCopy->Empty(0) );
	hyp_CHECK_HR( m_pMeshCopy->AddMeshBuilder(m_pMesh,D3DRMADDMESHBUILDER_NOSUBMESHES) );
}


//CMesh
CMesh::CMesh() :
m_pMesh(0),
m_pFrame(0),
m_bVisible(false),
m_Color(1,1,1)
{
	
}

void CMesh::Create(CRenderTool& Render)
{
	hyp_ASSERT(m_pMesh==0);
	hyp_ASSERT(m_pFrame==0);

	hyp_CHECK_HR( Render.GetD3DRM()->CreateFrame(Render.GetRootFrame(), &m_pFrame) );
	hyp_CHECK_HR( Render.GetD3DRM()->CreateMeshBuilder(&m_pMesh) );	
	
}

void CMesh::Visible(void)
{
	if(m_bVisible==false) {
		for(int i=0;i<3;i++) { hyp_CHECK_HR( m_pFrame->AddVisual(m_pMesh) ); }
		m_bVisible=true;
	}
}

void CMesh::Unvisible(void)
{
	if(m_bVisible==true) {
		for(int i=0;i<3;i++) { hyp_CHECK_HR( m_pFrame->DeleteVisual(m_pMesh) ); }
		m_bVisible=false;
	}
}

bool CMesh::IsVisible(void) const
{
	return m_bVisible;
}

Color CMesh::GetColor(void) const
{
	return m_Color;
}

void CMesh::SetColor(const Color& color)
{
	m_Color=color;
	
	hyp_ASSERT(m_pMesh);
	hyp_CHECK_HR( m_pMesh->SetColor(m_Color.GetColorRGB()) );
}

//CDXMesh
void CDXMesh::SetFile(std::string FilePath)
{
	m_FilePath=FilePath;
}
void CDXMesh::Reset(void)
{
	m_pMesh->Empty(0);
	
	hyp_CHECK_HR( m_pMesh->Load((void*)GetLocalPath(m_FilePath).c_str(),
		NULL,
		D3DRMLOAD_FROMFILE,
		NULL,
		NULL) );

	SetColor(m_Color);
}

//CHypMesh
CHypMesh::CHypMesh() :
m_ColorFixed(1,0,0),
m_bApplyFixedColor(true)
{

}

void CHypMesh::Create(CRenderTool& RenderTool)
{
	CMesh::Create(RenderTool);
	hyp_CHECK_HR( RenderTool.GetD3DRM()->CreateMaterial(D3DVALUE(8.0),&m_pMaterial) );	
}

void CHypMesh::SetColorFixed(const Color& color)
{
	m_ColorFixed=color;

	SetColor(m_Color);
}

Color CHypMesh::GetColorFixed() const
{
	return m_ColorFixed;
}

void CHypMesh::SetHypMesh(hyp_ker::IUnknown* pHypMesh)
{
	m_pHypMesh=pHypMesh;
}

//FIXTODO : merger IContainer et IContainerKey
void CHypMesh::SetColor(const Color& color)
{
	CMesh::SetColor(color);

	if(m_bApplyFixedColor) {

	hyp_fem::t_spFEOMesh spMesh=m_pHypMesh;
	hyp_fem::t_spGeometricObject spObject=spMesh;
	hyp_fem::t_spContainer spContSides=spObject->GetContainerOfSides();

	for(int i=0;i<spContSides->GetSize();i++) {
		Color fColor;
		IDirect3DRMFace2* pFace=0;
		hyp_CHECK_HR( m_pMesh->GetFace(i,&pFace) );

		if(spMesh->IsSideFixed(i,0)) {
			fColor.m_R=m_ColorFixed.m_R;			
		}
		if(spMesh->IsSideFixed(i,1)) {
			fColor.m_G=m_ColorFixed.m_G;			
		}
		if(spMesh->IsSideFixed(i,2)) {
			fColor.m_B=m_ColorFixed.m_B;		
		}

		if( spMesh->IsSideFixed(i,0) || spMesh->IsSideFixed(i,1) || spMesh->IsSideFixed(i,2) ) {
			hyp_CHECK_HR( pFace->SetColor(fColor.GetColorRGB()) );
			hyp_CHECK_HR( m_pMaterial->SetEmissive(fColor.m_R,fColor.m_G,fColor.m_B) );
			hyp_CHECK_HR( pFace->SetMaterial(m_pMaterial) );
		}
	}
	
	} else {
		hyp_CHECK_HR(m_pMaterial->SetEmissive(0.0f,0.0f,0.0f) );
	}
}

void CHypMesh::ApplyFixedColor(bool bApply)
{
	m_bApplyFixedColor=bApply;
	SetColor(m_Color);
}

bool CHypMesh::ApplyFixedColor(void) const
{
	return m_bApplyFixedColor;
}

void CHypMesh::Reset(void)
{
	m_pMesh->Empty(0);

	try {
		hyp_out::t_spD3DMeshObject spD3DMesh;
		spD3DMesh.CreateInstance(hyp_out::CLSID_hypCObjectXFile,0);	
		spD3DMesh->SetObject(m_pHypMesh);
		spD3DMesh->InitializeD3DMeshBuilder(m_pMesh);
	} catch(hyp_ker::CComException hr) {
		hyp_FATAL( ("Error during the reset of the mesh") );
	}

	SetColor(m_Color);
}


//CMeshTool
CRenderTool::CRenderTool() :
m_DeviceQuality(D3DRMFILL_SOLID)
{
	for(int i=0;i<3;i++) {
		m_Axes[i].SetFile("axe.x");
	}	
}

void CRenderTool::CreateDrawClipper(void)
{
	hyp_ker::CSmartInterface<IDirect3DRM> spD3DRM1;
	hyp_CHECK_HR( Direct3DRMCreate( &spD3DRM1 ) );
	
	hyp_CHECK_HR( spD3DRM1->QueryInterface(IID_IDirect3DRM3, (void **)&m_pD3DRM) );
	hyp_CHECK_HR( m_pD3DRM->SetOptions(D3DRMOPTIONS_RIGHTHANDED) );

	hyp_CHECK_HR( DirectDrawCreateClipper(0, &m_pDDClipper, NULL) );
	hyp_CHECK_HR( m_pDDClipper->SetHWnd( 0, GetHViewWindow() ) );

    hyp_CHECK_HR( m_pD3DRM->CreateFrame( NULL, &m_pScene ) );
	hyp_CHECK_HR( m_pD3DRM->CreateFrame(m_pScene, &m_pCamera) );	
}

void CRenderTool::CreateDeviceAndView(DWORD& dwWidth, DWORD& dwHeight)
{
	if( !dwWidth || !dwHeight)
		hyp_THROW_COM(hyp_ker::com_false);

	// Create the D3DRM device from this window and using the specified D3D
    // driver.
	GUID* pGUID=0;
    hyp_CHECK_HR( m_pD3DRM->CreateDeviceFromClipper(m_pDDClipper,pGUID,
		dwWidth,dwHeight,&m_pDevice) );

	// Create the D3DRM viewport using the camera frame.  Set the background
    // depth to a large number.  The width and height may be slightly
    // adjusted, so get them from the device to be sure.
    dwWidth  = m_pDevice->GetWidth();
    dwHeight = m_pDevice->GetHeight();
    
	hyp_CHECK_HR( m_pD3DRM->CreateViewport( m_pDevice, m_pCamera, 0, 0,
		                                       dwWidth, dwHeight,
											   &m_pViewport ) );
    m_pViewport->SetBack( 5000.0f );	

}

void CRenderTool::ResetCameraPosition(void)
{
	hyp_ASSERT( m_pCamera );
	    
    hyp_CHECK_HR( m_pCamera->SetPosition(m_pScene,
                                  D3DVAL(0),
                                  D3DVAL(-10),
                                  D3DVAL(60)) );
	hyp_CHECK_HR( m_pCamera->SetOrientation(m_pScene,
                                  D3DVAL(0),
                                  D3DVAL(0),
                                  D3DVAL(-1),
								  D3DVAL(0),
								  D3DVAL(0),
								  D3DVAL(0)) );

}

void CRenderTool::BuildScene(void)
{
	ResetCameraPosition();	
	
	// Local varaibles
	hyp_ker::CSmartInterface<IDirect3DRMFrame3> spLights;
	hyp_ker::CSmartInterface<IDirect3DRMLight> spLight1;
    hyp_ker::CSmartInterface<IDirect3DRMLight> spLight2;
	


	
    // Create a light frame as a child of the scene frame
    hyp_CHECK_HR( m_pD3DRM->CreateFrame(m_pScene, &spLights) );

    // Position the light frame within the scene
    hyp_CHECK_HR( spLights->SetPosition(m_pScene,
                                 D3DVAL(0),
                                 D3DVAL(-15),
                                 D3DVAL(15)) );
/*	hyp_CHECK_HR( spLights->SetOrientation(m_pScene,
                                  D3DVAL(0),
                                  D3DVAL(0),
                                  D3DVAL(0),
								  D3DVAL(0),
								  D3DVAL(0),
								  D3DVAL(0)) );*/
    
    // Create a bright, parallel point light
    // Color values are as follows:
    // 0.0 = totally dim and 1.0 = totally bright
    hyp_CHECK_HR( m_pD3DRM->CreateLightRGB(D3DRMLIGHT_POINT,
                             D3DVAL(0.5),   // Red intensity
                             D3DVAL(0.5),   // Green intensity
                             D3DVAL(0.6),   // Blue intensity
                             &spLight1) );
    
    // Add light to light frame
    hyp_CHECK_HR( m_pCamera->AddLight(spLight1) );
    
    // Create a dim, ambient light and attach it to the scene 
    // frame,
    hyp_CHECK_HR( m_pD3DRM->CreateLightRGB(D3DRMLIGHT_AMBIENT, 
                                       D3DVAL(0.2),    // red
                                       D3DVAL(0.2),    // green
                                       D3DVAL(0.2),    // blue
                                       &spLight2) );  
    hyp_CHECK_HR( m_pScene->AddLight(spLight2) );

	//le cube->construit au chargement du document
	m_Body.Create(*this);	
	
	for(int i=0;i<3;i++) {
		m_Axes[i].Create(*this);
		m_Axes[i].Reset();		
	}
	hyp_CHECK_HR( m_Axes[0].GetMesh()->SetColorRGB(1.0,0,0) );
	hyp_CHECK_HR( m_Axes[1].GetMesh()->SetColorRGB(0,1.0,0) );
	hyp_CHECK_HR( m_Axes[2].GetMesh()->SetColorRGB(0,0,1.0) );

	hyp_CHECK_HR( m_Axes[0]->SetOrientation(m_pScene,
                              D3DVAL(0),
                              D3DVAL(0),
                              D3DVAL(-1),
							  D3DVAL(1),
							  D3DVAL(0),
							  D3DVAL(0)) );
							  
	hyp_CHECK_HR( m_Axes[1]->SetOrientation(m_pScene,
                              D3DVAL(-1),
                              D3DVAL(0),
                              D3DVAL(0),
							  D3DVAL(0),
							  D3DVAL(1),
							  D3DVAL(0)) );

	hyp_CHECK_HR( m_Axes[2]->SetOrientation(m_pScene,
                              D3DVAL(0),
                              D3DVAL(-1),
                              D3DVAL(0),
							  D3DVAL(0),
							  D3DVAL(0),
							  D3DVAL(0)) );


}

void CRenderTool::RenderScene(void)
{
	hyp_CHECK_HR( m_pDevice->SetQuality(m_DeviceQuality+D3DRMSHADE_FLAT+D3DRMLIGHT_ON) );
	
	// Clear the viewport.
	hyp_CHECK_HR( m_pViewport->Clear(D3DRMCLEAR_ALL) );

    // Render the aScene to the viewport.
	hyp_CHECK_HR( m_pViewport->Render(m_pScene) );

    // Update the window.
    hyp_CHECK_HR( m_pDevice->Update() );
}

void CRenderTool::ResetBody(void)
{
	m_Body.SetHypMesh(GetViewDocument()->m_spUnknownMesh);
	m_Body.Reset();
	m_Body.Visible();
}

void CRenderTool::SetAxesVisible(void)
{
	for(int i=0;i<3;i++) {
		m_Axes[i].Visible();
	}
}

void CRenderTool::SetAxesUnvisible(void)
{
	for(int i=0;i<3;i++) {
		m_Axes[i].Unvisible();
	}
}

bool CRenderTool::IsAxesVisible(void)
{
	hyp_ASSERT( (m_Axes[0].IsVisible()==m_Axes[1].IsVisible()) && 
		(m_Axes[0].IsVisible()==m_Axes[2].IsVisible()) );
	return m_Axes[0].IsVisible();
}

void CRenderTool::SetColorBody(const Color& color,const Color& color_fixed)
{
	m_Body.SetColor(color);
	m_Body.SetColorFixed(color_fixed);
}

void CRenderTool::GetColorBody(Color& color,Color& color_fixed) const
{
	color=m_Body.GetColor();
	color_fixed=m_Body.GetColorFixed();
}

hyp_NAMESPACE_END_HYPVISUAL