/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypVisual.
 HypVisual is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

class CHypVisualView;

class CTool {
static CHypVisualView* m_pView;
protected:
	static CHypVisualDoc* GetViewDocument(void);
	static HWND GetHViewWindow(void);
	static void InvalidateView(void);
	static void UpdateStatusBar(void);
public:
	static void SetView(CHypVisualView* pView) { m_pView=pView; }	
};

class CRotationTool :
protected CTool 
{
	UINT m_Key;
	bool m_bRotate;
		
public:
	CRotationTool();	
	void RotateBegin(UINT);
	void RotateEnd(UINT);

	inline bool IsRotated(void) const { return m_bRotate; }
	inline operator bool() const { return IsRotated(); }
};

class CRenderTool;

class CSimulationTool :
protected CTool
{
public:
	typedef double (*PTR_FUNCTION)(const double&,const double&);
	static double SinusFunction(const double&,const double&);
	static double LinearFunction(const double&,const double&);
	static double SinusExpFunction(const double&,const double&);
	static int ms_NumberOfOscillation;
	static double ms_Damping;
	

	enum e_Simulation { play, stop, pause };
	
	CSimulationTool();
	~CSimulationTool();
	void CreateCopyMesh(CRenderTool&);

	inline e_Simulation GetState(void) const { return m_State; }

	void Stop(void);
	void Play(void);
	void Pause(void);
	void Start(void);

	void SetInterpolationParameters(bool,PTR_FUNCTION,double,int,int);
	void GetInterpolationParameters(BOOL&,PTR_FUNCTION&,double&,int&,int&) const;
	void SetInterpolationData(IDirect3DRMMeshBuilder3*,const t_real_array&);
private:
	static void CALLBACK TimeProc(UINT,UINT,DWORD,DWORD,DWORD);
	static t_real ms_StepsDone;

	void KillTimer(void);
	MMRESULT m_TimeID;
	int m_Delay; 
	e_Simulation m_State;

	//pour l'interpolation
	PTR_FUNCTION m_InterpolFunction;
	t_real_array m_Displacement;
	IDirect3DRMMeshBuilder3* m_pMesh;
	int m_NumberOfSteps;
	double m_MultiCoef;
	bool m_bRepeat;
	hyp_ker::CSmartInterface<IDirect3DRMMeshBuilder3> m_pMeshCopy;	//copie du cube

};

class CRenderTool;

class CMesh
{
protected:
	hyp_ker::CSmartInterface<IDirect3DRMFrame3> m_pFrame;
	hyp_ker::CSmartInterface<IDirect3DRMMeshBuilder3> m_pMesh;

	bool m_bVisible;
	
	Color m_Color;

public:
	CMesh();
	inline IDirect3DRMFrame3* operator->() { return m_pFrame; }
	IDirect3DRMMeshBuilder3* GetMesh(void) { return m_pMesh; }
	
	virtual void Create(CRenderTool&);
	virtual void Reset(void) =0;
	
	void Visible(void);
	void Unvisible(void);
	bool IsVisible(void) const;
	
	Color GetColor(void) const;
	virtual void SetColor(const Color&);
};

class CDXMesh :
public CMesh
{
	std::string m_FilePath;
public:
	void SetFile(std::string);
	virtual void Reset(void);
};

class CHypMesh :
public CMesh
{
	hyp_ker::IUnknown* m_pHypMesh;
	Color m_ColorFixed;
	hyp_ker::CSmartInterface<IDirect3DRMMaterial2> m_pMaterial;
	bool m_bApplyFixedColor;

public:
	CHypMesh();
	void SetHypMesh(hyp_ker::IUnknown*);
	virtual void SetColor(const Color&);
	virtual void Reset(void);
	virtual void Create(CRenderTool&);

	void SetColorFixed(const Color&);
	Color GetColorFixed(void) const;
	void ApplyFixedColor(bool);
	bool ApplyFixedColor(void) const;
};



class CRenderTool :
protected CTool
{
	int m_DeviceQuality;

	hyp_ker::CSmartInterface<IDirect3DRM3> m_pD3DRM; // D3DRM object
	hyp_ker::CSmartInterface<IDirectDrawClipper> m_pDDClipper; // DDrawClipper object
	hyp_ker::CSmartInterface<IDirect3DRMDevice3> m_pDevice; // D3DRM device 
	hyp_ker::CSmartInterface<IDirect3DRMViewport2> m_pViewport; // D3DRM viewport

	hyp_ker::CSmartInterface<IDirect3DRMFrame3> m_pScene; // Root frame for the scene
	hyp_ker::CSmartInterface<IDirect3DRMFrame3> m_pCamera; // Camera's frame
	
	CDXMesh m_Axes[3];
	CHypMesh m_Body;

public:
	inline IDirect3DRMFrame3* GetRootFrame(void) {
		return m_pScene;
	}
	inline IDirect3DRMFrame3* GetCameraFrame(void) {
		return m_pCamera;
	}
	inline IDirect3DRM3* GetD3DRM(void) {
		return m_pD3DRM;
	}

	void ResetBody(void);
	void SetColorBody(const Color&,const Color&);
	void GetColorBody(Color&,Color&) const;
	bool ApplyFixedColorBody(void) const { return m_Body.ApplyFixedColor(); }
	void ApplyFixedColorBody(bool bApply) { m_Body.ApplyFixedColor(bApply); }
	IDirect3DRMMeshBuilder3* GetBodyMesh(void) { return m_Body.GetMesh(); }

	void SetAxesVisible(void);
	void SetAxesUnvisible(void);
	bool IsAxesVisible(void);

	CRenderTool();
	
	void CreateDrawClipper(void);
	void CreateDeviceAndView(DWORD&,DWORD&);
	void BuildScene(void);
	void RenderScene(void);
	void ResetCameraPosition(void);

	inline int GetDeviceQuality(void) const { return m_DeviceQuality; }
	inline void SetDeviceQuality(int quality) { m_DeviceQuality=quality; }
};
