/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 HypVisual.
 HypVisual is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include "hyp.visual.intern.h"


hyp_NAMESPACE_BEGIN_HYPVISUAL


IMPLEMENT_DYNCREATE(CHypVisualView, CView)

BEGIN_MESSAGE_MAP(CHypVisualView, CView)
	//{{AFX_MSG_MAP(CHypVisualView)
	ON_WM_CREATE()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(IDM_OPTIONS, OnOptionsDialogShow)
	ON_COMMAND(IDM_VISCOSITY, OnSetViscosity)
	ON_COMMAND(ID_BEGINSIMULATION, OnSimulationBegin)
	ON_COMMAND(ID_REORIENTBODY, OnReorientMesh)
	ON_COMMAND(ID_RESETBODY, OnResetPosition)
	ON_COMMAND(ID_STOPSIMULATION, OnSimulationStop)
	ON_COMMAND(ID_UPDATE_STATUSBAR,OnUpdateStatusBar)
	ON_UPDATE_COMMAND_UI(ID_STOPSIMULATION, OnUpdateSimulationStop)
	ON_UPDATE_COMMAND_UI(ID_BEGINSIMULATION, OnUpdateSimulationBegin)
	ON_UPDATE_COMMAND_UI(ID_SIMULATION_FORCES, OnUpdateSetForces)
	ON_UPDATE_COMMAND_UI(ID_SHOWDETAILS_CLEARFORCES, OnUpdateClearForces)
	ON_UPDATE_COMMAND_UI(IDM_VISCOSITY, OnUpdateSetViscosity)	
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()





/////////////////////////////////////////////////////////////////////////////
// CHypVisualView construction/destruction

CHypVisualView::CHypVisualView()
{
	m_Param.DeplacDiff=D3DVALUE(0.5);	
	m_Param.RotationDiff=D3DVALUE(0.1);


}

CHypVisualView::~CHypVisualView()
{
}




//------------------------------------------------------------------------------------
//Draw Functions
void CHypVisualView::OnDraw(CDC* pDC)
{
	//CHypVisualDoc* pDoc = GetDocument();
	//ASSERT_VALID(pDoc);
/*
	//on regarde si on est en simulation
	if(CallBackArg.m_bInterpolationFinished) {
		if(m_GeometricInterpolationArg.m_bRepeat) {
			OnResetPosition();
			m_CallBackArg.m_bInterpolationFinished=false;
		} else {
			m_Simulation.Stop();
		}
	}
	*/

	m_Render.RenderScene();
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
//Keyboard Functions
void CHypVisualView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	D3DVECTOR pos;
	m_Render.GetCameraFrame()->GetPosition(NULL,&pos);
	
	D3DVECTOR rot;
	rot.x=rot.y=rot.z=0.0f;
	
	
	m_Rotation.RotateBegin(nChar);

	switch (nChar) {
	case VK_DOWN:
		if(m_Rotation)	rot.y=1;
		else pos.y+=m_Param.DeplacDiff;			
		break;
    case VK_UP:
		if(m_Rotation) rot.y=-1;
		else pos.y-=m_Param.DeplacDiff;
		break;
    case VK_LEFT:
		if(m_Rotation) rot.x=1;
		else pos.x+=m_Param.DeplacDiff;
		break;
    case VK_RIGHT:
		if(m_Rotation)	rot.x=-1;
		else pos.x-=m_Param.DeplacDiff;		
		break;
	case VK_ADD:
		if(m_Rotation) rot.z=-1;
		else pos.z-=m_Param.DeplacDiff;
		break;
    case VK_SUBTRACT:
		if(m_Rotation) rot.z=1;
		else pos.z+=m_Param.DeplacDiff;
		break;
	default:
		goto label_DefaultProc;
	}
	
	if(m_Rotation) {
		D3DRMVectorNormalize(&rot);
		hyp_CHECK_HR( m_Render.GetCameraFrame()->AddRotation(D3DRMCOMBINE_AFTER,
			rot.x,rot.y,rot.z,m_Param.RotationDiff) );
	} else {
		hyp_CHECK_HR( m_Render.GetCameraFrame()->SetPosition(m_Render.GetRootFrame(),pos.x,pos.y,pos.z) );
	}
	
	

	Invalidate(FALSE);	

label_DefaultProc:		
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CHypVisualView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	m_Rotation.RotateEnd(nChar);

	CView::OnKeyUp(nChar, nRepCnt, nFlags);
}
//------------------------------------------------------------------------------------


void CHypVisualView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	//FIXTODO : pourquoi cela marche pas dans OnCreate (state bar not initialized?)
	m_Simulation.Stop();
	m_Render.ResetBody();
	

	Invalidate(FALSE);
}

int CHypVisualView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	CTool::SetView(this);

	m_Render.CreateDrawClipper();
    //FIXTODO
	DWORD dwWidth=1000,dwHeight=1000;
	m_Render.CreateDeviceAndView(dwWidth,dwHeight);
	m_Render.BuildScene();

	m_Render.SetAxesVisible();

	m_Simulation.CreateCopyMesh(m_Render);


	return 0;
}

BOOL CHypVisualView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	lpszClassName=::AfxRegisterWndClass(0,
		::AfxGetApp()->LoadStandardCursor(IDC_CROSS),
		(HBRUSH) GetStockObject(BLACK_BRUSH),
		0);
	
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

//------------------------------------------------------------------------------------
//Dialog Functions
void CHypVisualView::OnOptionsDialogShow() 
{
	COptionsDialog aDlg;
	aDlg.m_FillMode=m_Render.GetDeviceQuality();
	m_Render.GetColorBody(aDlg.m_Color,aDlg.m_ColorFixed);
	aDlg.m_bShowAxes=m_Render.IsAxesVisible();
	aDlg.m_ApplyFixedColor=m_Render.ApplyFixedColorBody();

	if(aDlg.DoModal()==IDOK) {
		m_Render.SetDeviceQuality(aDlg.m_FillMode);
		m_Render.SetColorBody(aDlg.m_Color,aDlg.m_ColorFixed);
		aDlg.m_bShowAxes ? m_Render.SetAxesVisible() : m_Render.SetAxesUnvisible();
		m_Render.ApplyFixedColorBody(aDlg.m_ApplyFixedColor==TRUE);
	}
	Invalidate(FALSE);	
}

void CHypVisualView::OnSetViscosity() 
{
	CViscosityDialog aDlg;
	m_Simulation.GetInterpolationParameters(aDlg.m_bRepeat,
		aDlg.m_Function,
		aDlg.m_MultiCoef,
		aDlg.m_NumberOfSteps,
		aDlg.m_Delay);
	aDlg.m_NumberOfOscillation=CSimulationTool::ms_NumberOfOscillation;
	aDlg.m_Damping=CSimulationTool::ms_Damping;	

	if(aDlg.DoModal()==IDOK) {
		m_Simulation.SetInterpolationParameters( (aDlg.m_bRepeat==TRUE ? true : false),
										aDlg.m_Function,
										aDlg.m_MultiCoef,
										aDlg.m_NumberOfSteps,
										aDlg.m_Delay);
		CSimulationTool::ms_NumberOfOscillation=aDlg.m_NumberOfOscillation;
		CSimulationTool::ms_Damping=aDlg.m_Damping;
	}
}
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
//Simulation Functions
void CHypVisualView::OnSimulationBegin() 
{
	switch(m_Simulation.GetState()) {
	case CSimulationTool::stop: {
		
		GetDocument()->m_spMeshMesh->Render();
		t_real_array _displacement=GetDocument()->m_spMeshMesh->GetGlobalDisplacement();

		m_Simulation.SetInterpolationData(m_Render.GetBodyMesh(),_displacement);

		m_Simulation.Start();
		
	}
	break;

	case CSimulationTool::play:
		m_Simulation.Pause();
		break;
	case CSimulationTool::pause:
		m_Simulation.Play();
		break;
	default:
		hyp_ASSERT(0);
	}
	
}

void CHypVisualView::OnUpdateSimulationBegin(CCmdUI* pCmdUI) 
{
	switch(m_Simulation.GetState()) {
	case CSimulationTool::play:
		pCmdUI->SetText("Pause\tF5");
		break;
	case CSimulationTool::pause:
		pCmdUI->SetText("Resume\tF5");
		break;
	case CSimulationTool::stop:
		pCmdUI->SetText("Play\tF5");
	}
}

void CHypVisualView::OnSimulationStop() 
{
	m_Simulation.Stop();	
}

void CHypVisualView::OnUpdateSimulationStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_Simulation.GetState()!=CSimulationTool::stop);
}

void CHypVisualView::OnUpdateSetForces(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(m_Simulation.GetState()==CSimulationTool::stop);
}

void CHypVisualView::OnUpdateClearForces(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(m_Simulation.GetState()==CSimulationTool::stop);
}

void CHypVisualView::OnUpdateSetViscosity(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(m_Simulation.GetState()==CSimulationTool::stop);
}

void CHypVisualView::OnResetPosition() 
{
	CWaitCursor wait;
	m_Simulation.Stop();
	m_Render.ResetBody();
	Invalidate(FALSE);
}


void CHypVisualView::OnReorientMesh()
{
	m_Render.ResetCameraPosition();
	Invalidate(NULL);
}

void CHypVisualView::OnUpdateStatusBar()
{
	switch(m_Simulation.GetState()) {
	case CSimulationTool::play:
		GetFrame()->ShowPlayState();
		break;
	case CSimulationTool::pause:
		GetFrame()->ShowPauseState();
		break;  
	case CSimulationTool::stop:
		GetFrame()->ShowStopState();
		break;
	default:
		hyp_ASSERT(0);
	}
	

	GetFrame()->ShowIsRotated(m_Rotation.IsRotated());
}

LRESULT CHypVisualView::DefWindowProc( UINT message, WPARAM wParam, LPARAM lParam )
{
	//FIXTODO
	if(message==ID_STOPSIMULATION) {
		OnSimulationStop();
	}

	return CView::DefWindowProc(message,wParam,lParam);
}

hyp_NAMESPACE_END_HYPVISUAL
